;--------------------------------------------------------------------------------
;
; NAME:     read_scanline_v3.pro
;
; PURPOSE:  Example program for reading HIRS clear-sky scanline data.
;
; LANGUAGE: IDL
;
; CALLING SEQUENCE:
;                   From IDL prompt, type -> read_scanline_v2
;
; INPUTS:
;          Prompted for input when executed
;  Requires functions local_zenith.pro and hlimb.pro
;
; OUTPUT:
;           xtime..........GMT Time in seconds
;           xlon...........longitude in degrees East from GM (0-359.99)
;           xlat...........latitude in degrees (-90S to 90N)
;           xszen..........solar zenith angle (0 to 90 -> 90 is night)
;           xalt...........satellite altitude in km
;           iline..........line number
;           ispot..........scan spot position (1-56)
;           ref............reflectance (0 - 1 valid; 2.55 and 2.54 missing)
;           tb(19).........no limb bright. temp. in Kelvin.  Array index
;                          matches channel number.
;           tblc(19).......limb corrected bright. temp. in Kelvin
;
;
; NOTES:  
;         1. Limb correction coefficient files are needed if limb correction
;            is performed.
;         2. Program accounts for byte swapping if ran on big endian computer.
;         3. Limb correction coefficients are function of channel,satellite,
;            angle, and solar zenith.  Daytime observations for the 4.3
;            micron channels (13-19) are not limb corrected.
;
; CREATION HISTORY:
;                   - Darren Jackson CIRES/ETL May 2003
;                   - Version 3 uses little endian regression
;                     coefficients  Darren Jackson July 2005
;-----------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------
;
; NAME:  local_zenith
;
; PURPOSE:
;          Computes local zenith angle for HIRS observations (output in radians)
;
; INPUTS:
;         alt,isp
;         alt -> satellite altitude in km
;         isp -> scan position
;
; OUTPUTS:
;         lza -> local zenith angle in radians
;
; CREATION HISTORY:
;                    Darren Jackson CIRES/ETL August 2001
;------------------------------------------------------------------------------------

FUNCTION local_zenith, alt, isp, lza

scan_angle=-49.5+1.8*FLOAT(isp-1)
sinzen=ABS((alt+6371.)*SIN(scan_angle*!dtor)/6371.)
lza=ASIN(sinzen)
RETURN, lza
END
;---------------------------------------------------------------------------------
;
; NAME: hlimb
;
; PURPOSE:  Limb correction routine for the HIRS clear-sky data.  Called by
;           read_hirs_cs.pro
;
; LANGUAGE: IDL
;
; INPUTS:
;         szen  -> solar zenith angle (0 - 90 degrees)
;         nchn2 -> number of HIRS IR channels (night)
;         nchn3 -> number of HIRS IR channels (day)
;         ispot -> scan spot location (1-56)
;         xalt  -> altitude in km
;         rcoef -> array of regression coefficients for limb correction (night)
;         chnord -> array of channel predictors (night)
;         npred  -> array of number of predictors (night)
;         rcoef2 -> array of regression coefficients for limb correction (day)
;         chnord2 -> array of channel predictors (day)
;         npred2  -> array of number of predictors (day)
;         tb     -> vector of brightness temperatures in Kelvin
;
; OUTPUTS:
;         tblc   -> vector of limb corrected brightness temperatures (Kelvin)
;
; CREATION HISTORY:
;                   Darren Jackson CIRES/ETL August 2001
;----------------------------------------------------------------------------------

FUNCTION hlimb, szen, nchn2, nchn3, ispot, xalt, rcoef, chnord, npred, $
rcoef2, chnord2, npred2, tb, tblc

cosx = [1.0, 0.9, 0.8, 0.7, 0.6, 0.5]
y = FLTARR(2)
tblc=FLTARR(nchn2)

lza     = LOCAL_ZENITH(xalt, ispot)
coslzen = COS(lza)
indx1   = FIX(10.*(1.-coslzen))
indx2   = indx1 + 1

if(szen ge 90.) then begin

 for i=0,nchn2-1 do begin
  y(0)=0.
  y(1)=0.
  for j=1,npred(indx1,i)-1 do begin
     y(0)=y(0)+rcoef(j,indx1,i)*tb(chnord(j-1,indx1,i)-1)
  endfor
  y(0)=y(0)+rcoef(0,indx1,i)
  for j=1,npred(indx2,i)-1 do begin
     y(1)=y(1)+rcoef(j,indx2,i)*tb(chnord(j-1,indx2,i)-1)
  endfor
  y(1)=y(1)+rcoef(0,indx2,i)
               
;  Interpolate to find limb value
  slope=(y(1)-y(0))/(cosx(indx2)-cosx(indx1))
  tblc(i)=y(1)+slope*(coslzen-cosx(indx2))
 endfor

endif else begin

 for i=0,nchn3-1 do begin
  y(0)=0.
  y(1)=0.
  for j=1,npred(indx1,i)-1 do begin
     y(0)=y(0)+rcoef(j,indx1,i)*tb(chnord(j-1,indx1,i)-1)
  endfor
  y(0)=y(0)+rcoef(0,indx1,i)
  for j=1,npred(indx2,i)-1 do begin
     y(1)=y(1)+rcoef(j,indx2,i)*tb(chnord(j-1,indx2,i)-1)
  endfor
  y(1)=y(1)+rcoef(0,indx2,i)
               
;  Interpolate to find limb value
  slope=(y(1)-y(0))/(cosx(indx2)-cosx(indx1))
  tblc(i)=y(1)+slope*(coslzen-cosx(indx2))
 endfor

endelse

return, tblc

end

;
;---------------------------------
; BEGIN MAIN PROGRAM
;---------------------------------

PRO read_scanline_v2

;
; Request input parameters
;
satid=''
year2=''
jday=''
limb=''
little=''
r_path=''
i_path=''
iprint=''

read,satid,prompt='Enter Satellite ID number (12 for NOAA-12):'
read,year2,prompt='Enter 2 digit year (98 for 1998):'
read,jday,prompt='Enter day of year (1-366):
read,limb,prompt='Limb correction? (y or n)
if(limb eq 'y') then read,r_path,prompt='Directory path for reg. coefficients:'
read,little,prompt='Using little endian computer? (y or n)'
read,i_path,prompt='HIRS scanline data path directory:'
read,iprint,prompt='Print to 1 - screen, 2 - file: (1 or 2)'

;satid='12'
;year2='98'
;jday='32'
;limb='y'     ; y - limb correction,   n - no limb correction
;little='n'   ; y - little Endian (INTEL or DEC), n - all other systems
;r_path='/data/dlj/tovs1b/limb/regcoef/'    ; regression coefficients location
;i_path='../cs_data/'                       ; HIRS clear-sky data location

; regression coefficient parameters (do not change)

nchn2=19
nchn3=12
ncoef=15
nang=6

; HIRS scan line input data

itime=0L
ilon=0
ilat=0
iszen=0
ialt=0
iline=0
isp=byte(0)
isf=byte(0)
iref=byte(0)
itb=intarr(nchn2)

; HIRS scan line output data

tblc=fltarr(nchn2)
tb=fltarr(nchn2)

; Read regression coefficient files

if(limb eq 'y') then begin
  icoef=lonarr(ncoef,nang,nchn2)
  chnord=lonarr(ncoef-1,nang,nchn2)
  npred=lonarr(nang,nchn2)
  icoef2=lonarr(ncoef,nang,nchn3)
  chnord2=lonarr(ncoef-1,nang,nchn3)
  npred2=lonarr(nang,nchn3)

  rfile=r_path+'hirs_limb_coef_n'+satid+'.lon.dat'
  rfile2=r_path+'hirs_limb_coef_n'+satid+'.lon.lir.dat'
  if(little eq 'n') then begin
    openr,r1lun,rfile,/swap_endian,/get_lun
    openr,r2lun,rfile2,/swap_endian,/get_lun
  endif else begin
    openr,r1lun,rfile,/get_lun
    openr,r2lun,rfile2,/get_lun
  endelse
  readu,r1lun,icoef
  rcoef=icoef/100000.
  free_lun,r1lun
  readu,r2lun,icoef2
  rcoef2=icoef2/100000.
  free_lun,r2lun

  cfile=r_path+'hirs_limb_chnord_n'+satid+'.dat'
  cfile2=r_path+'hirs_limb_chnord_n'+satid+'.lir.dat'
  if(little eq 'n') then begin
    openr,c1lun,cfile,/swap_endian,/get_lun
    openr,c2lun,cfile2,/swap_endian,/get_lun
  endif else begin
    openr,c1lun,cfile,/get_lun
    openr,c2lun,cfile2,/get_lun
  endelse
  readu,c1lun,chnord
  free_lun,c1lun
  readu,c2lun,chnord2
  free_lun,c2lun

  nfile=r_path+'hirs_limb_npred_n'+satid+'.dat'
  nfile2=r_path+'hirs_limb_npred_n'+satid+'.lir.dat'
  if(little eq 'n') then begin
    openr,n1lun,nfile,/swap_endian,/get_lun
    openr,n2lun,nfile2,/swap_endian,/get_lun
  endif else begin
    openr,n1lun,nfile,/get_lun
    openr,n2lun,nfile2,/get_lun
  endelse
  readu,n1lun,npred
  free_lun,n1lun
  readu,n2lun,npred2
  free_lun,n2lun
endif
;
; Define input file name and open
;
if(jday lt 10) then infile='HIRS.N'+satid+'.Y'+year2+'.D00'+jday+'.V22.CS'
if(jday ge 10 and jday lt 100) then infile='HIRS.N'+satid+'.Y'+year2+'.D0'+jday+'.V22.CS'
if(jday ge 100) then infile='HIRS.N'+satid+'.Y'+year2+'.D'+jday+'.V22.CS'

if(little eq 'n') then begin
   openr,ilun,i_path+'/'+infile,/swap_endian,/get_lun
endif else begin
   openr,ilun,i_path+'/'+infile,/get_lun
endelse
;
; Open output file
;
if(iprint eq 2) then openw,olun,'read_scanline2.dat',/get_lun
;
; Read data, write ouput data
;
while not eof(ilun) do begin
  readu,ilun,itime,ilon,ilat,iline,isp,iszen,ialt,isf,iref,itb
  xtime=itime/100.
  xlon=(ilon/100.)+180.
  xlat=ilat/100.
  xszen=iszen/100.
  xalt=ialt/10.
  ispot=isp
  line=iline
  ref=iref/100.
  for k=0,nchn2-1 do tb(k)=(itb(k)/100.)+100.
;
; Call limb correction routine
;
  if(limb eq 'y') then begin
    tblc=hlimb(xszen,nchn2,nchn3,ispot,xalt,rcoef,chnord, $
               npred,rcoef2,chnord2,npred2,tb)
    if(xszen lt 90.) then tblc(12:18)=tb(12:18)
  endif
;
; Output results
;
  if(iprint eq 1) then begin
    print, xtime,xlon,xlat,xszen,xalt,line,ispot
    if(limb eq 'y') then begin
      for n=0,nchn2-1 do print, 'channel=',n+1,tb(n),tblc(n)
    endif else begin
       for n=0,nchn2-1 do print, 'channel=',n+1,tb(n)
    endelse
    print, format='(a16,4x,f6.2)', 'channel=     20',ref
    print, ' '
  endif
  if(iprint eq 2) then begin
    printf, olun, xtime,xlon,xlat,xszen,xalt,line,ispot
    if(limb eq 'y') then begin
      for n=0,nchn2-1 do printf, olun, 'channel=',n+1,tb(n),tblc(n)
    endif else begin
       for n=0,nchn2-1 do printf, olun, 'channel=',n+1,tb(n)
    endelse
    printf, olun, format='(a13,4x,f6.2)', 'channel=     20',ref
    printf, olun, ' '
  endif

endwhile
free_lun,ilun
if(iprint eq 2) then free_lun,olun
if(iprint eq 2) then print,'Output data in file read_scanline2.dat'

end
;------------------------------------------------
;   END MAIN PROGRAM
;------------------------------------------------
