/* clearlc2nc.c */
/* 
#--------------------------------------------------------------------------------
#@***p* HIRS/clearlc2nc.c
#
# NAME
#   clearlc2nc.c
#
# LOCATION
#   .../hirs/scripts/2nc/clearlc2nc.c
#
# PURPOSE
#   Convert HIRS limb-corrected clear-sky data to NetCDF format
#
# DESCRIPTION
#   Convert HIRS limb-corrected clear_sky data to NetCDF format
#
# AUTHOR
#   L. Shi
#
# CREATION DATE
#   Aug 2005
#
# COPYRIGHT
#   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
#   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
#   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
#   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
#   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
#   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
#   TO USERS.
#
# MODIFICATION HISTORY
#   May 2010 - D. Wunder put in robodoc format
#
# INPUTS
#   HIRS.$SATID.Y$YY.*.LC = limb-corrected HIRS data, in which 
#                           SATID = satellite id; YY = two digit year
#
# OUTPUTS
#   HIRS.$SATID.Y$YY.*.nc = data converted to netCDF format
#
# EXTERNALS
#   libnetcdf.a
#
# TODO
#   n/a
#
# LANGUAGE
#   C
#
#@*****
#---------------------------------------------------------------------------------
*/

#include <stdio.h>
#include <stdlib.h>
#include "netcdf.h"

#define total 200000
#define ch 19

void
check_err(const int stat, const int line, const char *file) {
    if (stat != NC_NOERR) {
	   (void) fprintf(stderr, "line %d of %s: %s\n", line, file, nc_strerror(stat));
        exit(1);
    }
}

int
main(argc, argv) 
     int argc;
     char **argv;
{			/* create netcdf file */

  FILE         *ifp;                /* Input file pointer */
  static float tch[ch][total],tlc[ch][total],visdata[total];
  static float second[total],londata[total],latdata[total];
  static float zendata[total],altdata[total];
  static int   linedata[total],posidata[total],test;
  int          n,i,k,i1;
  float        f1,f2;
  char         s[100];
  static int   count;

  ifp = fopen(argv[1],"r");
  for (n=0; n<total; n++) {
    test=fscanf(ifp, "%s%f%f%f%f%f%d%d",
		s,&second[n],&londata[n],&latdata[n],
                &zendata[n],&altdata[n],&linedata[n],&posidata[n]);
    if (test == -1)
      goto endofdata;
    for (k=0; k<ch; k++) {
      fscanf(ifp, "%s%d%f%f", s,&i1,&tch[k][n],&tlc[k][n]);
    }
    fscanf(ifp, "%s%d%f", s,&i1,&visdata[n]);
    count++;
  }
 endofdata: printf("count=%d\n", count);
  fclose(ifp);

   int  ncid;			/* netCDF id */

   /* dimension ids */
   int ch_dim;
   int time_dim;

   /* dimension lengths */
   size_t ch_len = ch;
   size_t time_len = NC_UNLIMITED;

   /* variable ids */
   int bt_id;
   int lc_id;
   int vis_id;
   int lat_id;
   int lon_id;
   int time_id;
   int zen_id;
   int alt_id;
   int line_id;
   int posi_id;

   /* rank (number of dimensions) for each variable */
#  define RANK_bt 2
#  define RANK_lc 2
#  define RANK_vis 1
#  define RANK_lat 1
#  define RANK_lon 1
#  define RANK_time 1
#  define RANK_zen 1
#  define RANK_alt 1
#  define RANK_line 1
#  define RANK_posi 1

   /* variable shapes */
   int bt_dims[RANK_bt];
   int lc_dims[RANK_lc];
   int vis_dims[RANK_vis];
   int lat_dims[RANK_lat];
   int lon_dims[RANK_lon];
   int time_dims[RANK_time];
   int zen_dims[RANK_zen];
   int alt_dims[RANK_alt];
   int line_dims[RANK_line];
   int posi_dims[RANK_posi];

   /* attribute vectors */
   double lat_valid_range[2];
   double lon_valid_range[2];

   /* enter define mode */
   int stat = nc_create("hirsbt_lc.nc", NC_CLOBBER, &ncid);
   check_err(stat,__LINE__,__FILE__);

   /* define dimensions */
   stat = nc_def_dim(ncid, "ch", ch_len, &ch_dim);
   check_err(stat,__LINE__,__FILE__);
   stat = nc_def_dim(ncid, "time", time_len, &time_dim);
   check_err(stat,__LINE__,__FILE__);

   /* define variables */

   bt_dims[0] = time_dim;
   bt_dims[1] = ch_dim;
   stat = nc_def_var(ncid, "bt", NC_FLOAT, RANK_bt, bt_dims, &bt_id);
   check_err(stat,__LINE__,__FILE__);

   lc_dims[0] = time_dim;
   lc_dims[1] = ch_dim;
   stat = nc_def_var(ncid, "lc", NC_FLOAT, RANK_lc, lc_dims, &lc_id);
   check_err(stat,__LINE__,__FILE__);

   vis_dims[0] = time_dim;
   stat = nc_def_var(ncid, "vis", NC_FLOAT, RANK_vis, vis_dims, &vis_id);
   check_err(stat,__LINE__,__FILE__);

   lat_dims[0] = time_dim;
   stat = nc_def_var(ncid, "lat", NC_FLOAT, RANK_lat, lat_dims, &lat_id);
   check_err(stat,__LINE__,__FILE__);

   lon_dims[0] = time_dim;
   stat = nc_def_var(ncid, "lon", NC_FLOAT, RANK_lon, lon_dims, &lon_id);
   check_err(stat,__LINE__,__FILE__);

   time_dims[0] = time_dim;
   stat = nc_def_var(ncid, "time", NC_FLOAT, RANK_time, time_dims, &time_id);
   check_err(stat,__LINE__,__FILE__);

   zen_dims[0] = time_dim;
   stat = nc_def_var(ncid, "zen", NC_FLOAT, RANK_zen, zen_dims, &zen_id);
   check_err(stat,__LINE__,__FILE__);

   alt_dims[0] = time_dim;
   stat = nc_def_var(ncid, "alt", NC_FLOAT, RANK_alt, alt_dims, &alt_id);
   check_err(stat,__LINE__,__FILE__);

   line_dims[0] = time_dim;
   stat = nc_def_var(ncid, "line", NC_INT, RANK_line, line_dims, &line_id);
   check_err(stat,__LINE__,__FILE__);

   posi_dims[0] = time_dim;
   stat = nc_def_var(ncid, "posi", NC_INT, RANK_posi, posi_dims, &posi_id);
   check_err(stat,__LINE__,__FILE__);

   /* assign attributes */
   stat = nc_put_att_text(ncid, bt_id, "long_name", 22,
                          "brightness temperature");
   check_err(stat,__LINE__,__FILE__);
   stat = nc_put_att_text(ncid, bt_id, "units", 6, "Kelvin");
   check_err(stat,__LINE__,__FILE__);
   stat = nc_put_att_text(ncid, bt_id, "coordinates", 7, "lon lat");
   check_err(stat,__LINE__,__FILE__);
   stat = nc_put_att_text(ncid, lc_id, "long_name", 37,
                          "limb-corrected brightness temperature");
   check_err(stat,__LINE__,__FILE__);
   stat = nc_put_att_text(ncid, lc_id, "units", 6, "Kelvin");
   check_err(stat,__LINE__,__FILE__);
   stat = nc_put_att_text(ncid, lc_id, "coordinates", 7, "lon lat");
   check_err(stat,__LINE__,__FILE__);
   stat = nc_put_att_text(ncid, vis_id, "long_name", 15,
                          "HIRS channel 20");
   check_err(stat,__LINE__,__FILE__);
   stat = nc_put_att_text(ncid, vis_id, "units", 1, "%");
   check_err(stat,__LINE__,__FILE__);
   stat = nc_put_att_text(ncid, vis_id, "coordinates", 7, "lon lat");
   check_err(stat,__LINE__,__FILE__);

   stat = nc_put_att_text(ncid, lat_id, "units", 13, "degrees_north");
   check_err(stat,__LINE__,__FILE__);
   lat_valid_range[0] = -90;
   lat_valid_range[1] = 90;
   stat = nc_put_att_double(ncid, lat_id, "valid_range", NC_DOUBLE, 2, 
                            lat_valid_range);
   check_err(stat,__LINE__,__FILE__);
   stat = nc_put_att_text(ncid, lon_id, "units", 12, "degrees_east");
   check_err(stat,__LINE__,__FILE__);
   lon_valid_range[0] = 0;
   lon_valid_range[1] = 360;
   stat = nc_put_att_double(ncid, lon_id, "valid_range", NC_DOUBLE, 2,
                            lon_valid_range);
   check_err(stat,__LINE__,__FILE__);

   stat = nc_put_att_text(ncid, time_id, "units", 7, "seconds");
   check_err(stat,__LINE__,__FILE__);

   stat = nc_put_att_text(ncid, time_id, "long_name", 8, "seconds from 0 UTC");
   check_err(stat,__LINE__,__FILE__);

   stat = nc_put_att_text(ncid, zen_id, "long_name", 6, "zenith");
   check_err(stat,__LINE__,__FILE__);

   stat = nc_put_att_text(ncid, alt_id, "long_name", 8, "altitude");
   check_err(stat,__LINE__,__FILE__);

   stat = nc_put_att_text(ncid, line_id, "long_name", 4, "line");
   check_err(stat,__LINE__,__FILE__);

   stat = nc_put_att_text(ncid, posi_id, "long_name", 8, "position");
   check_err(stat,__LINE__,__FILE__);
   /*
   stat = nc_put_att_text(ncid, NC_GLOBAL, "source", 22, "HIRS Output");
   check_err(stat,__LINE__,__FILE__);
   */
   /* leave define mode */
   stat = nc_enddef (ncid);
   check_err(stat,__LINE__,__FILE__);

   /* store bt, lc, and vis*/
   static size_t bt_start[RANK_bt];
   static size_t bt_count[RANK_bt];
   static float bt[ch*total];
   static size_t lc_start[RANK_lc];
   static size_t lc_count[RANK_lc];
   static float lc[ch*total];
   static size_t vis_start[RANK_lat];
   static size_t vis_count[RANK_lat];
   static float vis[total];

   for (n=0; n<count; n++) {
     for (k=0; k<ch; k++) {
       i = ch*n + k;
       bt[i] = tch[k][n];
       lc[i] = tlc[k][n];
     }
     vis[n] = visdata[n];
   }
   time_len = count;
   bt_start[0] = 0;
   bt_start[1] = 0;
   bt_count[0] = time_len;
   bt_count[1] = ch_len;
   stat = nc_put_vara_float(ncid, bt_id, bt_start, bt_count, bt);
   check_err(stat,__LINE__,__FILE__);
   lc_start[0] = 0;
   lc_start[1] = 0;
   lc_count[0] = time_len;
   lc_count[1] = ch_len;
   stat = nc_put_vara_float(ncid, lc_id, lc_start, lc_count, lc);
   check_err(stat,__LINE__,__FILE__);
   vis_start[0] = 0;
   vis_count[0] = time_len;
   stat = nc_put_vara_float(ncid, vis_id, vis_start, vis_count, vis);
   check_err(stat,__LINE__,__FILE__);

   /* store time,lat,lon,zen,alt,line,posi */
   static size_t time_start[RANK_time];
   static size_t time_count[RANK_time];
   static float time[total];
   static size_t lat_start[RANK_lat];
   static size_t lat_count[RANK_lat];
   static float lat[total];
   static size_t lon_start[RANK_lon];
   static size_t lon_count[RANK_lon];
   static float lon[total];
   static size_t zen_start[RANK_zen];
   static size_t zen_count[RANK_zen];
   static float zen[total];
   static size_t alt_start[RANK_alt];
   static size_t alt_count[RANK_alt];
   static float alt[total];
   static size_t line_start[RANK_line];
   static size_t line_count[RANK_line];
   static int line[total];
   static size_t posi_start[RANK_posi];
   static size_t posi_count[RANK_posi];
   static int posi[total];

   for (n=0; n<count; n++) {
     time[n] = second[n];
     lat[n] = latdata[n];
     lon[n] = londata[n];
     zen[n] = zendata[n];
     alt[n] = altdata[n];
     line[n] = linedata[n];
     posi[n] = posidata[n];
   }
   time_len = count;
   time_start[0] = 0;
   time_count[0] = time_len;
   stat = nc_put_vara_float(ncid, time_id, time_start, time_count, time);
   check_err(stat,__LINE__,__FILE__);
   lat_start[0] = 0;
   lat_count[0] = time_len;
   stat = nc_put_vara_float(ncid, lat_id, lat_start, lat_count, lat);
   check_err(stat,__LINE__,__FILE__);
   lon_start[0] = 0;
   lon_count[0] = time_len;
   stat = nc_put_vara_float(ncid, lon_id, lon_start, lon_count, lon);
   check_err(stat,__LINE__,__FILE__);
   zen_start[0] = 0;
   zen_count[0] = time_len;
   stat = nc_put_vara_float(ncid, zen_id, zen_start, zen_count, zen);
   check_err(stat,__LINE__,__FILE__);
   alt_start[0] = 0;
   alt_count[0] = time_len;
   stat = nc_put_vara_float(ncid, alt_id, alt_start, alt_count, alt);
   check_err(stat,__LINE__,__FILE__);
   line_start[0] = 0;
   line_count[0] = time_len;
   stat = nc_put_vara_int(ncid, line_id, line_start, line_count, line);
   check_err(stat,__LINE__,__FILE__);
   posi_start[0] = 0;
   posi_count[0] = time_len;
   stat = nc_put_vara_int(ncid, posi_id, posi_start, posi_count, posi);
   check_err(stat,__LINE__,__FILE__);

   stat = nc_close(ncid);
   check_err(stat,__LINE__,__FILE__);
   return 0;
}
