#!/bin/csh
#-------------------------------------------------------------------------------------
#@***p* HIRS/create_subdir2
#
# NAME
#   create_subdir2
#
# LOCATION
#   /hirsoperation/clearsky4/scripts/create_subdir2
#
# PURPOSE
#   Moves C2, CH and CS data files into monthly subdirectories.
#
# DESCRIPTION
#   Moves C2, CH and CS data files into monthly subdirectories.
#   THE CALLING SEQUENCE: 
#       create_subdir2 <data type> <satellite number> <begin month number> 
#                      <begin 4 digit year> <end month> <end year>
#
# AUTHOR
#   Darren Jackson CIRES/ETL  
#
# CREATION DATE
#   April 2003 
#
# COPYRIGHT
#   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
#   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
#   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
#   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
#   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
#   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
#   TO USERS.
#
# MODIFICATION HISTORY
#   Mar 2010 - D. Wunder put in robodoc format
#   Jun 2003 - Darren Jackson, Version 2 loops over months and years.
#
# INPUTS
#   type    ->  Data type for CH,CS,C2 files
#   sat     ->  NOAA satellite number (5,6,7,...17)
#   bmonth  ->  Begin month number (1-12)
#   byear   ->  Four digit begin year
#   emonth  ->  End month
#   eyear   ->  End year
#
# OUTPUTS
#   None (files moved to new directoties)
# 
# EXTERNALS
#   date2calday.csh
#   two_digit_year.csh
#
# TODO
#   Move data paths out of home directory
#
# LANGUAGE
#   C-Shell
#
#@*****
#-------------------------------------------------------------------------------------

# Define inputs
set type   = $1
set sat    = $2
set bmonth = $3
set byear  = $4
set emonth = $5
set eyear  = $6

# Define script directory
set scrdir = /home/lshi/clearsky/scripts

# Set path names
switch ($type)
   case C2:
      set dpath = "/home/lshi/clearsky/data/c2_data/"
      breaksw
   case CH:
      set dpath = "/home/lshi/clearsky/data/ch_data/"
      breaksw
   case CS:
      set dpath = "/home/lshi/clearsky/data/cs_data/"
      breaksw
endsw

#  Loop through each year
@ iyr = $byear
while ($iyr <= $eyear)

   if($iyr == $byear) then
      @ imn = $bmonth
   else
      @ imn = 1
   endif
   if($iyr == $eyear) then
      @ emn = $emonth
   else
      @ emn = 12
   endif

# Find two digit year
   set iyr2 = `${scrdir}/two_digit_year.csh $iyr`

#  Loop through each month
   while ($imn <= $emn)

# Find begin and end calender day
      set bcalday = `${scrdir}/date2calday.csh $iyr $imn 1`
      if($imn < 12) then
        @ next_month = $imn + 1
        @ next_year = $iyr
        set ecalday = `${scrdir}/date2calday.csh $next_year $next_month 1`
        @ ecalday --
      else
        set ecalday = `${scrdir}/date2calday.csh $iyr $imn 31`
      endif

      set mon_dir = `printf "%1s%02d%2s%02d%2s%02d" N ${sat} .Y ${iyr2} .M ${imn}`

# Create directories if they don't exist
      set mpath = ${dpath}$mon_dir
      if(! -e $mpath) mkdir $mpath

# Move files to new directories
         cd $mpath

switch ($type)
         case C2:
            @ calday = $bcalday
            while($calday <= $ecalday)
            foreach angle ('10' '09' '08' '07')
               set fname = `printf "%6s%02d%2s%02d%2s%03d%2s%2s%3s" HIRS.N $sat .Y $iyr2 .D $calday .A $angle .C2`
               if(-e $dpath/$fname) mv $dpath/$fname $mpath
            end
            @ calday ++
         end
         breaksw

         case CH:
            @ calday = $bcalday
            while($calday <= $ecalday)
              foreach gmt ('06' '18')
              foreach angle ('10' '09' '08' '07')
                 set fname = `printf "%6s%02d%2s%02d%2s%03d%1s%2s%3s%2s%3s" HIRS.N $sat .Y $iyr2 .D $calday . $gmt Z.A $angle .CH`
                 if(-e $dpath/$fname) mv $dpath/$fname $mpath
              end
            end
            @ calday ++
         end
         breaksw

         case CS:
         @ calday = $bcalday
         while($calday <= $ecalday)
           set fname = `printf "%6s%02d%2s%02d%2s%03d%7s" HIRS.N $sat .Y $iyr2 .D $calday .V22.CS`
           if(-e $dpath/$fname) mv $dpath/$fname $mpath
           @ calday ++
         end
         breaksw
      endsw

      @ imn ++
   end

   @ iyr ++
end
