#!/bin/csh -v
#---------------------------------------------------------------------------------
#***p* HIRS/hirsclearskyproc_m2.csh
#
# NAME
#  hirsclearskyproc_m2.csh
#
# LOCATION
#  hirsoperation/clearsky4/scripts/hirsclearskyproc_m2.csh
#
# PURPOSE
#  To process HIRS 1b data into HIRS clear-sky swath data set.
#
# DESCRIPTION
#   Reads HIRS 1b data in following steps:
#   1.  Grabs 1 day of HIRS 1b orbital files from remote directory
#   2.  Uses ITPP/AAPP to produce HIRS non-limb corrected TBs
#   3.  Runs quality control program on all-sky swath data
#   4.  Applies limb correction for channel 8
#   5.  Applies cloud detection
#   6.  Outputs HIRS clear-sky swath data set into daily files.
# 
# AUTHOR
#  Darren Jackson CIRES/ETL
#
# CREATION DATE
#  November 2004
#
# COPYRIGHT
#  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
#  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
#  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
#  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
#  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
#  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
#  TO USERS.
#
# MODIFICATION HISTORY
#  March 2010 - D. Wunder put in robodoc format
#
# INPUTS
#   bjday            ->  start day of year (1-366) 
#   ejday            ->  end day of year (1-366)
#   byear            ->  4 digit start year
#   eyear            ->  4 digit end year
#   satid            ->  Satellite number of data to be processed
#   remove_tmp_files ->  diagnostic tool.  Set to n to see all intermediate files.
#   copyx            ->  y - copies fortran executables each day from bin directory. 
#                        n - does not.
########################

set lastMonth = `date -d "last month" +%m`
set thisMonth = `date +%m`
set lastMonthYear = `date -d "last month" +%Y`
set thisMonthYear = `date +%Y`

#set bjday    = 221
set bjday = `date -d "$lastMonthYear-$lastMonth-01" +%-j` 
#set ejday    = 250
set ejday = `date -d "$thisMonthYear-$thisMonth-15" +%-j`
#set byear    = 2009
set byear = $lastMonthYear
#set eyear    = 2009
set eyear = $thisMonthYear

set satid    =  2
set infileprefix = nss.hirx.m2.d
set remove_tmp_files = n            # y - yes, n - no
set copyx            = y            # y = yes, n = no


########################
# NOTES   
#   - uses an improved limb correction technique that does not
#     use coefficients from channels 13-19.
#
#   - uses ITPP preprocessor for HIRS/2 data and 
#     AAPP preprocessor for HIRS/3 ingestion
#
#
# EXTERNALS
#  SCRIPTS:
#   julian1969_to_date.....Finds year, month, day given julian day from 1/1/69
#   date_to_julian1969.....Finds julian day from 1/1/69 given year,month,day
#   read_atov..............Runs the AAPP processing code and produces 1c formatted file
#
#  ITPP Fortran executables:
#   ITPP executables from the ITPP v5.20 code.  These routines have been
#   modified to produce different output parameters that are used in this code
#
#   hirsix4253_Hv2........Shortens HIRS 1b data record
#   hirsix4256_Hv2........Shortens HIRS 1b data record
#   tovinw3_Hv2...........Applies calibration to HIRS data (does not use MSU data)
#   tovprw7_Hv2...........Computes bidirectional refl., writes all parameters
#                         needed for processing.
#
#  Pathfinder Fortran executables:
#
#   HIRS Pathfinder executables
#
#   hirs1c................Applies QC, limb correction to chn 8, creates
#                         daily tb max/vis refl min grids, outputs orbit mean
#                         statistics, pixel level all-sky data.
#   hcloud................Applies space and time contrast test using all-sky
#                         data and grids from hirs1c for cloud detection.
#   multi_day_grid........Combines daily max/min TB grids into longer time periods.
#   hcloudcomp............Produces clear-sky TB 5-day grid by using 5-,15-, and
#                         35-day composite grids from multi_day_grid.
#   hcloud_lt.............Performs second cloud detection test using long-term
#                         TB statistics from hcloudcomp.  Output from this file
#                         is the final HIRS clear-sky swath data.
#
#
# VARIABLES
#
#  tape           Number of tapes to process (only used 1 so far)
#  daycnt         Counts number of days processed.  Starts with 1
#  day            Day number (1-366) of current day being processed from tape
#  year           2 digit year number of current day being processed from tape
#  readfile       1 for successful read of tape.  2 to 3 for tar read errors. 
#  name           list of HIRS 1b file names for the current day being processed.
#  year4          4 digit year number of current day being processed from tape
#  jday69         julian day number of current day being processed (referenced from 1/1/69)
#  fname          List of current day HIRS 1b files without ?? station designation
#  cnt            Counter used to name HIRS output files from ITPP processing
#  input          name used in foreach loop of fname list
#  orb            orbit id number given to each 1b file
#  jday69_m1      julian day before jday69 (m1 = minus 1)
#  jday69_m1_prv  julian day previous to jday69_m1.  Not always 1 less than jday69_m1 if
#                 missing day occurs on tape or read errors occur with tape
#  prv_yyddd      list containing two digit year [1] and day of year [3] for jday69_m1
#  rem            remainder to test for end of pentad (5-day) period 
#  bjday69        begin day of 5-,15- or 35-day period for compute HIRS8 grid statistics
#  ejday69        end day of 5-,15- or 35-day period for compute HIRS8 grid statistics
#  jday_17        julian day that is at center of 35-day period for hcloudcomp grid statistics
#  jday_34        julian day that is 34 days less than current day being processed for clouds/cs
#  jday_30        """""""""""""""""""30""""""""""""""""""""""""""""""""""""""""""""""""""""""""
#  jday_24        """""""""""""""""""24""""""""""""""""""""""""""""""""""""""""""""""""""""""""
#  jday_10        """""""""""""""""""10""""""""""""""""""""""""""""""""""""""""""""""""""""""""
#  yr_mn_day      Finds year, month, and day for julian day parameters shown above
#  bday           begin day taken from yr_mn_day
#  eday           end day taken from yr_mn_day
#  gfile          Name of temporary grid file to be removed
#  biyr2          begin year (2 digit) used for name of temporary grid file
#  eiyr2          end year (2 digit) used for name of temporary grid file
#  jday19         julian day used to process the clear-sky and cloud swath data.
#  yr_mon_day     finds year,month,day for julian day defined by jday19
#  yr2            2 digit year from yr_mon_day
#  mon            2 digit month from yr_mon_day
#  yr_day         gives year and day of year from jday19
#  file           used to construct file name
#  cs_file        clear-sky swath full path file name
#  tfile          Temporary file name used to remove files
#
#
# FILES
#  tar.out                 Gives output results of tar read
#  tar_error.out           Gives location on tape where tar error occurred
#  tovprw7_Hv2.input       Input parameters for ITPP executable tovprw7_Hv2
#  tmp.list                List of current day HIRS 1b files without ?? station designation
#  hirsix.out              Output from the hirsix42**_Hv2 ITPP program
#  tovinw3.out             Output from the tovinw3_Hv2 ITPP program
#  tovprw7.out             Output from the tovprw7_Hv2 ITPP program
#  tmp3.list               Output of the orbit id numbers from all files for processing day
#  hirs_**.dat             Output TB data from ITPP.  ** are numbered from 1 to ~20 for one day
#  tov_itpp_save.out       Output information about ITPP processing errors
#  nss.*                   1b data file names
#  hirs1c.input/out        Input/output files needed for hirs1c
#  hcloud.input/out          Input/Output files for hcloud
#  multi_day_grid.input/out  Input/Output files for multi_day_grid
#  hcloud_lt.input/out       Input/Output files for hcloud_lt    
#
# Directory name description:
#  data_l    parent directory on lhotse where data sets and executables reside during processing
#  itpp      directory where ITPP processing code resides
#  src       directory location of source code and scripts uses to run processing
#  hirs_BIN_DIR  directory where all fortran executables are found
#  hirs_SCRIPT_DIR   directory where all processing scripts are found
#  bin_dir   directory where all executables are run from during processing
#  log_dir   directory where all log files are written
#  grid_dir  directory where grid data during processing are placed
#  tmp_dir   directory where all-sky and short term cloud swath data reside
#  orb_dir   directory where orbit statistics are placed.
#  itpp_bin  directory where all the ITPP executables live
#  itpp_run  directory where all the ITPP data results reside
#  cs_dir    directory where the clear-sky swath data is placed during processing
#
#*****
#####################################################################
#               START OF SCRIPT
#####################################################################
############# Core path directories #################################
#####################################################################
echo "Start time for HIRS_CLEARSKY"
date
alias mv mv
alias rm rm

#source ./hirs_config.csh

#set hirs_root = /rsad/hirs
#set ftp_drop_dir = eclipse:/raid/ftp/pub
#set local_drop_dir = /net/crystal/san3/hirs
#set hirslc_nc_dir = /net/crystal/san3/raid3a/hirslc_nc

#set data_l   = ${hirs_root}/data
#set itpp     = ${hirs_root}/itpp/itpp5.20
#set aapp     = ${hirs_root}/aapp/current/read_atov
#set src      = ${hirs_root}
#
###################################################################################
############ Establish set of subdirectories in core directories #################
###################################################################################
#

#set HIRS 1B data dir
#set test_dir = ${hirs_HOME}/test/nb/hirs1b/M02
#set test_dir = /net/crystal/san2/nb/hirs1b/M02

#set bin2_dir = ${hirs_HOME}/bin
#set scr_dir  = ${hirs_HOME}/scripts
set bin_dir  = ${hirs_DAT_DIR}/process
set hirs_dir = ${hirs_DAT_DIR}/hirs1b
set log_dir  = ${hirs_DAT_DIR}/log
set grid_dir = ${hirs_DAT_DIR}/grid
set tmp_dir  = ${hirs_DAT_DIR}/tmp_data
set orb_dir  = ${hirs_DAT_DIR}/orbit
set cs_dir   = ${hirs_DAT_DIR}/cs_data
#set cs_dir   = ${hirs_hirscs_dir}
#set ch_dir   = ${hirs_DAT_DIR}/ch_data
set itpp_bin = ${hirs_ITPP_DIR}/bin
set itpp_run = ${hirs_ITPP_DIR}/run
#
#########################################################################
############### Make subdirectories for output data #####################
#########################################################################
#
if(! -e $orb_dir) mkdir $orb_dir
if(! -e $log_dir) mkdir $log_dir
if(! -e $hirs_dir) mkdir $hirs_dir
if(! -e $bin_dir) mkdir $bin_dir
if(! -e $tmp_dir) mkdir $tmp_dir
if(! -e $grid_dir) mkdir $grid_dir
if(! -e $cs_dir) mkdir $cs_dir
#if(! -e $ch_dir) mkdir $ch_dir

# change filenames to lower case
#cd $test_dir
cd $sv_data_dir
$hirs_SCRIPT_DIR/casechange_u2l.ksh
cd $hirs_SCRIPT_DIR

#
######################################################################
################### Looping each DLT tape ############################
######################################################################
#

@ daycnt = 1
@ day = 0
@ year = 0

######################################################################
##  Need code to fill one day of HIRS data in directory $hirs_dir ####
######################################################################

################
# Loop on year #
#################

@ iyr4 = $byear
while ($iyr4 <= $eyear)
   set iyr2 = `${hirs_SCRIPT_DIR}/two_digit_year.csh $iyr4`
#
###############
# Loop on day #
###############
#
   if($iyr4 == $byear) then
      @ jday = $bjday
   else
      @ jday = 1
   endif
   if($iyr4 == $eyear) then
      @ edy = $ejday
   else
      set tmp = `${hirs_SCRIPT_DIR}/calday2date.csh $iyr4 1`
      if($tmp[3] == 0) then
         @ edy = 365
      else
         @ edy = 366
      endif
   endif

   while($jday <= $edy)

#
# Copy one of data from test directory
#
#     cd $test_dir
     cd $sv_data_dir
#     set ifiles = `printf "%s%02d%03d%s" $infileprefix $iyr2 $jday .`
     if($iyr4 >= 2000) then
        set iyr1 = `${hirs_SCRIPT_DIR}/one_digit_year.csh $iyr4`
        set ifiles = `printf "%s%s%01d%03d%s" $infileprefix 0 $iyr1 $jday .`
     else
        set ifiles = `printf "%s%02d%03d%s" $infileprefix $iyr2 $jday .`
     endif
#     echo ifiles = $ifiles
     echo "##############################################"
	echo $ifiles
     cp ${ifiles}* $hirs_dir
	ls -l $hirs_dir
##########################################################################
############# Extract day-of-year and year of data files #################
##########################################################################

     cd $hirs_dir
     set name = `ls nss.hir* | head -n 1`
     set year = `echo $name | awk '$0 ~ /nss./ {printf "%s\n", substr($0, index($0,".d")+2, 2)}'`
     set day = `echo $name | awk '$0 ~ /nss./ {printf "%s\n", substr($0, index($0,".d")+4, 3)}'`
     set test0 = `echo $day | cut -c1-1`
     if ($test0 != 0) then
       set daynum = $day
     else
       set daynum = `echo $day | cut -c2-3`
       set test0 = `echo $daynum | cut -c1-1`
       if ($test0 == 0) then
         set daynum = `echo $day | cut -c2-2`
       endif
     endif

##################################################################
############ Compute Julian day from Jan 1, 1969 ################# 
##################################################################
#     echo before date_to_julian69.csh iyr4=$iyr4 daynum=$daynum
echo $iyr4 $daynum
  set jday69 = `${hirs_SCRIPT_DIR}/date_to_julian69.csh $iyr4 1 $daynum` 
#    echo jday69 = $jday69
########################################################################
########### sort files into daily loop and run ITPP/AAPP ###############
########################################################################

  ls -1 ${hirs_dir}/nss.hir*.d${year}${day}.* | grep -v "??" > ${bin_dir}/tmp.list
  rm ${bin_dir}/tmp3.list
  if(! -z ${bin_dir}/tmp.list) then
    set fname = `ls ${hirs_dir}/nss.hir*.d${year}${day}.* | grep -v "??"`
    @ cnt = 1
    foreach input ($fname)
	echo "#################################################################"
	echo $input
      set orb = `echo $input | awk '$0 ~ /nss./ {printf "%s\n", substr($0, index($0,".b")+2, 7)}'`

#########################################
##### Use AAPP for HIRS/3 1b data #######
#########################################

      if($satid > 1) then
        set input2 = `echo $input | awk '$0 ~ /nss./ {printf "%s\n", substr($0, index($0,"/nss")+1,42)}'`
        ${hirs_AAPP_DIR}/read_atov/read_atov $input2 hirs_${cnt}.dat -c >>&! ${log_dir}/aapp.out
	pwd
 	if(! -e hirs_${cnt}.dat || -z hirs_${cnt}.dat) then
          echo 'AAPP could not process file '$input
          rm hirs_${cnt}.dat
        else
          mv hirs_${cnt}.dat $bin_dir
          echo $orb >>! ${bin_dir}/tmp3.list
          @ cnt++
        endif
      else

#########################################
##### Use ITPP for HIRS/2 1b data #######
#########################################

        echo 'a' >! ${log_dir}/tovprw7_Hv2.input
        echo '1' >> ${log_dir}/tovprw7_Hv2.input
        echo '0' >> ${log_dir}/tovprw7_Hv2.input  # 0 - no limb, 1 - limb cor.
        echo '0' >> ${log_dir}/tovprw7_Hv2.input
        mv $input ${itpp_run}/hirstape.dat
        cd $itpp_run
        if($satid >= 11 && $iyr4 >= 1995) then
          ${itpp_bin}/hirsix4253_Hv2 >&! ${log_dir}/hirsix.out
        else
          ${itpp_bin}/hirsix4256_Hv2 >&! ${log_dir}/hirsix.out
        endif
        if(-z ${log_dir}/hirsix.out) then
          ${itpp_bin}/tovinw3_Hv2 >&! ${log_dir}/tovinw3.out
          if(-z ${log_dir}/tovinw3.out) then           
            ${itpp_bin}/tovprw7_Hv2 < ${log_dir}/tovprw7_Hv2.input >&! ${log_dir}/tovprw7.out
            if(-z ${log_dir}/tovprw7.out) then
              echo $orb >>! ${bin_dir}/tmp3.list
              mv ${itpp_run}/tovssnda.dat ${bin_dir}/hirs_${cnt}.dat
              @ cnt++
            else
              echo 'Error 3 in orbit no. '${orb}' for satellite '$satid' on day '$day' of year '$year >>&! ${log_dir}/tov_itpp_save.out
              cat ${log_dir}/tovprw7.out >>&! ${log_dir}/tov_itpp_save.out
            endif
          else
            echo 'Error 2 in orbit no. '${orb}' for satellite '$satid' on day '$day' of year '$year >>&! ${log_dir}/tov_itpp_save.out
            cat ${log_dir}/tovinw5.out >>&! ${log_dir}/tov_itpp_save.out
          endif
        else
          echo 'Error 1 in orbit no. '${orb}' for satellite '$satid' on day '$day' of year '$year >>&! ${log_dir}/tov_itpp_save.out
          cat ${log_dir}/hirsix.out >>&! ${log_dir}/tov_itpp_save.out 
        endif
      endif
    end
    echo "Time ITPP/AAPP processing ended "
    rm ${hirs_dir}/nss*
    date

########################################################################################
# Compute orbit statistics, quality control, all-sky data output., daily grid max file #
########################################################################################

    cd $bin_dir
    if(-e core) rm core
    @ cnt--
    if($cnt > 0) then
      echo $cnt >! hirs1c.input
      cat ${bin_dir}/tmp3.list >> hirs1c.input
      echo $day >> hirs1c.input
      echo $year >> hirs1c.input
      echo $satid >> hirs1c.input

#      if($copyx == 'y') cp ${hirs_BIN_DIR}/hirs1c .
# HIRS/2 and HIRS/3 have different nspot values due to different output
# formats between ITPP and AAPP. This results in different executable hirs1c.
#      if($satid < 15) then
#        cp ${hirs_BIN_DIR}/hirs1c_hirs2 hirs1c
#      else
        #cp ${hirs_BIN_DIR}/hirs1c_m2 hirs1c
        cp ${hirs_BIN_DIR}/hirs1c hirs1c
#      endif

      ./hirs1c >>&! hirs1c.out
      if(-e core) then
        echo 'Error in hirs1c for satellite '$satid' on day '$day' of year '$year
        stop
      endif
      if($remove_tmp_files == 'y') rm hirs*.dat
      echo "Time hirs1c ended for satellite ${satid}, year ${year}, day ${day}"
      date

##########################################
# Retrieve year and day for previous day #
##########################################

      if($daycnt == 1) then
         @ jday69_m1_prv = $jday69 - 1
      else	
         @ jday69_m1_prv = $jday69_m1 + 1
      endif
      @ jday69_m1 = $jday69 - 1
#      echo jday69_m1 = $jday69_m1
      set prv_yyddd = `${hirs_SCRIPT_DIR}/julian69_to_date.csh 1 $jday69_m1` 
#      echo after prv_yyddd

################################################################################
# Perform cloud detection on previous day, make short-term clear-sky data file #
################################################################################

      echo $satid >! hcloud.input
      echo $prv_yyddd[1] >> hcloud.input
      echo $prv_yyddd[3] >> hcloud.input
 
      if($copyx == 'y') cp ${hirs_BIN_DIR}/hcloud .
      ./hcloud >>&! hcloud.out

      if(-e core) then
        echo "Error in hcloud for satellite '$satid' year ${prv_yyddd[1]}, day ${prv_yyddd[3]}"
        stop
      endif
      echo "Time hcloud ended for satellite ${satid}, year ${prv_yyddd[1]}, day ${prv_yyddd[3]}"
      date
    else
      echo 'ITPP errors for all orbits for satellite '$satid' on day '$day' of year '$year
    endif

  else
    echo 'No orbits available for satellite '$satid' on day '$day' of year '$year
  endif

####################################################################
########## Loop through days skipped due missing data ##############
####################################################################

  while($jday69_m1_prv <= $jday69_m1)

#
########################################################
#  This section computes 5-, 15-,35-day grid files     #
#  These grids required for long-term cloud detection  #
########################################################
#
#    echo jday69_m1_prv $jday69_m1_prv
    @ rem = $jday69_m1_prv % 5
    if($rem == 0) then
#
# Make 5-day Max grids files
#
#    echo bjday69 $bjday69
      @ bjday69 = $jday69_m1_prv - 4
      @ ejday69 = $jday69_m1_prv

      echo "5"      >! multi_day_grid.input
      echo $bjday69 >> multi_day_grid.input
      echo $ejday69 >> multi_day_grid.input
      echo $satid >> multi_day_grid.input
      if($copyx == 'y') cp ${hirs_BIN_DIR}/multi_day_grid .
      ./multi_day_grid >>&! multi_day_grid.out
      if(-e core) then
        echo "Error in multi_day_grid for satellite '$satid' on julian69 '$jday69_m1_prv'"
        stop
      endif
      echo "5-day multi_day_grid ended for satellite $satid, bjday ${bjday69}, ejday ${ejday69}"
      date
#
# Make 15-day Max grid files
#
      @ bjday69 = $jday69_m1_prv - 14
      @ ejday69 = $jday69_m1_prv

      echo "15"      >! multi_day_grid.input
      echo $bjday69 >> multi_day_grid.input
      echo $ejday69 >> multi_day_grid.input
      echo $satid >> multi_day_grid.input
      ./multi_day_grid >>&! multi_day_grid.out
      if(-e core) then
        echo "Error in multi_day_grid for satellite '$satid' on julian69 '$jday69_m1_prv'"
        stop
      endif
      echo "15-day multi_day_grid ended for satellite $satid, bjday ${bjday69}, ejday ${ejday69}"
      date
#
# Check for 35-day Max grid files
#
      @ bjday69 = $jday69_m1_prv - 34
      @ ejday69 = $jday69_m1_prv

      echo "35"      >! multi_day_grid.input
      echo $bjday69 >> multi_day_grid.input
      echo $ejday69 >> multi_day_grid.input
      echo $satid >> multi_day_grid.input
      ./multi_day_grid >>&! multi_day_grid.out
      if(-e core) then
        echo "Error in multi_day_grid for satellite '$satid' on julian69 '$jday69_m1_prv'"
        stop
      endif
      echo "35-day multi_day_grid ended for satellite $satid, bjday ${bjday69}, ejday ${ejday69}"
      date
#
# Make 5-day clear sky composite grid
#
      @ jday_17 = $jday69_m1_prv - 17
      echo $satid >! hcloudcomp.input
      echo $jday_17 >> hcloudcomp.input
      if($copyx == 'y') cp ${hirs_BIN_DIR}/hcloudcomp .
      ./hcloudcomp >>&! hcloudcomp.out
      if(-e core) then
        echo 'Error in hcloudcomp for satellite '$satid' on jday '$jday' of year '$year1
        stop
      endif
      echo "hcloudcomp ended for satellite $satid, jday_17 ${jday_17}"
      date
#
# Remove 5-day, 15-day and 35-day GRD files. 
#
      cd $grid_dir

      @ jday_34 = $jday69_m1_prv - 34
      @ jday_30 = $jday69_m1_prv - 30
      @ jday_24 = $jday69_m1_prv - 24
      @ jday_10 = $jday69_m1_prv - 10

#
# 5-day file
#
      if($remove_tmp_files == 'y') then
	
        set yr_mn_day = `${hirs_SCRIPT_DIR}/julian69_to_date.csh 1 $jday_34`
        set biyr2 = `${hirs_SCRIPT_DIR}/two_digit_year.csh $yr_mn_day`
        set bday  = $yr_mn_day[3]
        set yr_mn_day = `${hirs_SCRIPT_DIR}/julian69_to_date.csh 1 $jday_30`
        set eiyr2 = `${hirs_SCRIPT_DIR}/two_digit_year.csh $yr_mn_day`
        set eday  = $yr_mn_day[3]
        set gfile = `printf "%6s%02d%2s%02d%2s%03d%2s%02d%2s%03d%7s" HIRS.N $satid .Y $biyr2 .B $bday .Y $eiyr2 .E $eday .V22.GD`
        if(-e $gfile) then
          rm $gfile
          echo "Remove GRD file $gfile"
        endif
#
# 15-day file
#
        set yr_mn_day = `${hirs_SCRIPT_DIR}/julian69_to_date.csh 1 $jday_24`
        set biyr2 = `${hirs_SCRIPT_DIR}/two_digit_year.csh $yr_mn_day`
        set bday  = $yr_mn_day[3]
        set yr_mn_day = `${hirs_SCRIPT_DIR}/julian69_to_date.csh 1 $jday_10`
        set eiyr2 = `${hirs_SCRIPT_DIR}/two_digit_year.csh $yr_mn_day`
        set eday  = $yr_mn_day[3]
        set gfile = `printf "%6s%02d%2s%02d%2s%03d%2s%02d%2s%03d%7s" HIRS.N $satid .Y $biyr2 .B $bday .Y $eiyr2 .E $eday .V22.GD`
        if(-e $gfile) then
          rm $gfile
          echo "Remove GRD file $gfile"
        endif
#
# 35-day file
#
        set yr_mn_day = `${hirs_SCRIPT_DIR}/julian69_to_date.csh 1 $jday_34`
        set biyr2 = `${hirs_SCRIPT_DIR}/two_digit_year.csh $yr_mn_day`
        set bday  = $yr_mn_day[3]
        set yr_mn_day = `${hirs_SCRIPT_DIR}/julian69_to_date.csh 1 $jday69_m1_prv`
        set eiyr2 = `${hirs_SCRIPT_DIR}/two_digit_year.csh $yr_mn_day`
        set eday  = $yr_mn_day[3]
        set gfile = `printf "%6s%02d%2s%02d%2s%03d%2s%02d%2s%03d%7s" HIRS.N $satid .Y $biyr2 .B $bday .Y $eiyr2 .E $eday .V22.GD`
        if(-e $gfile) then
          rm $gfile
          echo "Remove GRD file $gfile"
        endif
      endif

    endif

#######################################################################
##########  Compute long-term clear-sky data ##########################
#######################################################################

#
#  Look back 4 pentads (19 days) from previous julian day
#
    @ jday19 = $jday69_m1_prv - 19
#
# Run hcloud_lt program.  This performs second cloud detection test and
# produces final clear-sky swath data.
#
    cd $bin_dir
    echo $satid >! hcloud_lt.input
    echo $jday19 >> hcloud_lt.input
 
    if($copyx == 'y') cp ${hirs_BIN_DIR}/hcloud_lt .
    ./hcloud_lt >>&! hcloud_lt.out

    if(-e core) then
      echo "Error in hcloud_lt for satellite '$satid' on jday '$jday19'"
      stop
    endif
    echo "hcloud_lt ended for satellite $satid, jday19 = ${jday19}"
    date

##############################
# Remove any temporary files #
##############################

    if($remove_tmp_files == 'y') then

      cd $tmp_dir
#
# Remove all-sky tmp file
#
      set tfile = `printf "%7s%02d%2s%02d%2s%03d%7s" HIRS.N $satid .Y $yr2 .D $yr_day[3] .V22.AS`
      if(-e $tfile) then
	rm $tfile
	echo "Remove file $tfile"
      endif	
#
# Remove short-term clear-sky tmp file
#
      set tfile = `printf "%7s%02d%2s%02d%2s%03d%8s" HIRS.N $satid .Y $yr2 .D $yr_day[3] .V22.CS1`
      if(-e $tfile) then
        rm $tfile
        echo "Remove file $tfile"
      endif
#
# Remove daily GRD files
#
      cd $grid_dir
      set gfile = `printf "%7s%02d%2s%02d%2s%03d%7s" HIRS.N $satid .Y $yr2 .D $yr_day[3] .V22.GD`
      if(-e $gfile) then
        rm $gfile
        echo "Removed file $gfile"
      endif

    endif

####################################################################
# End loop for days skipped due to tape read error or missing data #
####################################################################

    @ jday69_m1_prv ++
  end

##################################################
# Check day counter and move to next day on tape #
##################################################

  echo "Day counter = $daycnt ended on year $year, day $day"
  date
  echo " "
  echo " "

  @ daycnt ++
  @ jday ++
  end

@ iyr4 ++
 end


set lastMonthYear2d = `${hirs_SCRIPT_DIR}/two_digit_year.csh $lastMonthYear`

cd ${hirs_DAT_DIR}/tmp_data
cp HIRS.M0$satid.Y08*.V22.AS ${hirs_local_drop_dir}/hirs_as/M0$satid 
cp HIRS.M0$satid.Y09*.V22.AS ${hirs_local_drop_dir}/hirs_as/M0$satid 
cd ${hirs_DAT_DIR}/cs_data
cp HIRS.M0$satid.Y09*.V22.CS ${hirs_local_drop_dir}/hirs_cs/M0$satid 

${hirs_lcproc_dir}/work/runlimbcor_m.csh M0$satid 08
${hirs_lcproc_dir}/work/runlimbcor_m.csh M0$satid 09
${hirs_lcproc_dir}/work/runlimbcor_as.csh M0$satid 09

${hirs_SCRIPT_DIR}/2nc/hirslc2nc_m.csh M0$satid 08 
${hirs_SCRIPT_DIR}/2nc/hirslc2nc_m.csh M0$satid 09 
cd ${hirs_lcnc_dir}/LC_M0$satid 
#scp HIRS.M0$satid.Y08*.nc ${hirs_ftp_drop_dir}/hirslc/LC_M0$satid 
#scp HIRS.M0$satid.Y09*.nc ${hirs_ftp_drop_dir}/hirslc/LC_M0$satid 
#repeat this for all-sky(?) data

${hirs_SCRIPT_DIR}/mstep2/runmcalib.csh 
${hirs_SCRIPT_DIR}/mstep2/runtbbsatdaily2000.csh M0$satid 09 
${hirs_SCRIPT_DIR}/mstep2/runtbmonthly.csh M02 09 
${hirs_SCRIPT_DIR}/mstep2/runregiontb.csh 
####/net/crystal/home/lshi/mstep2/runch12ascombinedaily.csh 09 
${hirs_SCRIPT_DIR}/mstep2/runcombineprtch12.csh 

############################################################################

${hirs_SCRIPT_DIR}/hirstb/tb/tbdaily2000.csh M0$satid 08
${hirs_SCRIPT_DIR}/hirstb/tb/tbdaily2000.csh M0$satid 09
${hirs_SCRIPT_DIR}/hirstb/tb/tbmonthly.csh M0$satid 08
${hirs_SCRIPT_DIR}/hirstb/tb/tbmonthly.csh M0$satid 09
${hirs_SCRIPT_DIR}/hirstb/region/runregiontb.csh

#/net/crystal/home/lshi/step2/combinedailyleapproc.csh 09
#cd /net/crystal/san3/raid3b/step2/combine
#/net/crystal/san3/raid3b/step2/combine/ch12seriesdailyleap
#/net/crystal/home/lshi/ch12anom2nc/anom/anomproc.csh 09

#/net/crystal/home/lshi/asky/runch12ascalibproc.csh M0$satid 2009 2009
#/net/crystal/home/lshi/asky/rundailygridproc.csh M0$satid 2009 2009
#/net/crystal/home/lshi/asky/runch12ascombinedaily.csh 2009
# Currently the following concatenate an AS UTWV file 2007-2009
#/net/crystal/raid2a/lshi/datakeep/ch12intercalib/combine/ch12seriesdailyleap
#/net/crystal/askych12wavenc/anomproc.csh 09
# Currently the following concatenate an AS UTWV anomaly file 2007-2009
#/net/crystal/raid2a/lshi/datakeep/ch12intercalib/combine/fort_ieee/serieswave



