/*
#--------------------------------------------------------------------------------
#@***p* HIRS/tbmonthly.c
#
# NAME
#   tbmonthly.c
#
# LOCATION
#   .../hirs/scripts/mstep2/tbmonthly.c
#
# PURPOSE
#   Compute monthly grid and global mean Tb
#
# DESCRIPTION
#   Compute monthly grid and global mean Tb based on daily gridded data.
#
# AUTHOR
#   L. Shi
#
# CREATION DATE
#   Dec 2008
#
# COPYRIGHT
#   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
#   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
#   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
#   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
#   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
#   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
#   TO USERS.
#
# MODIFICATION HISTORY
#   May 2010 - D. Wunder put in robodoc format
#
# INPUTS
#   HIRS.$SATID.Y$YY.TB24 ... 24-hour daily gridded data for channels 1-12.
#
# OUTPUTS
#   mean.dat  ... gridded monthly Tb data for channels 1-12.
#   gmean.dat ... global mean Tb
#
# EXTERNALS
#   None
#
# TODO
#   n/a
#
# LANGUAGE
#   C
#
#@*****
#---------------------------------------------------------------------------------
*/

#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>
#include <math.h>

#define nlon 144
#define nlat 72
#define nch 12
#define minday 5   /* min days in a month that have available data */
#define nmon 12    /* 12 months */

main(argc, argv)
     int argc;
     char **argv;
{
  FILE       *ifp;                /* Input file pointer                */
  FILE       *ofp1,*ofp2;
  int        i,j,k,m,n;
  int num[nlat][nlon][nch];
  float tb[nch];
  float sum[nlat][nlon][nch],mean[nlat][nlon][nch];
  int sat, year, day;
  static int monnorm[nmon] = {31,59,90,120,151,181,212,243,273,304,334,365};
  static int monleap[nmon] = {31,60,91,121,152,182,213,244,274,305,335,366};
  static int nday[nmon];
  int nb,ne,testend;
  float gsum[nch],gmean[nch];
  int gnum[nch],yyyy,yy;
  char s[100];

  /* Determine if it is a leap year */
    ifp = fopen(argv[1],"r");
    sscanf(argv[1],"%10c%2d",s,&yy);
    if (yy > 50)
      yyyy = 1900 + yy;
    else
      yyyy = 2000 + yy;
    if (yyyy % 4 == 0)
      for (m=0; m<12; m++)
        nday[m] = monleap[m];
    else
      for (m=0; m<12; m++)
        nday[m] = monnorm[m];
    fclose(ifp);

  ofp1 = fopen("mean.dat","w");
  ofp2 = fopen("gmean.dat","w");

  for (m=0; m<nmon; m++) {
    for (k=0; k<nch; k++)
      for (i=0; i<nlat; i++)
        for (j=0; j<nlon; j++) {
          sum[i][j][k] = 0.0;
          num[i][j][k] = 0;
        }

    ifp = fopen(argv[1],"r");
    if (m==0)
      nb = 0;
    else
      nb = nday[m-1];
    ne = nday[m];
    for (n=0; n<366; n++) {
      testend = fscanf(ifp, "%d%d%d", &sat, &year, &day);
      if (testend == -1)
	goto endofdata;
      for (k=0; k<nch; k++)
        for (i=0; i<nlat; i++)
	  for (j=0; j<nlon; j++) {
	    fscanf(ifp, "%f", &tb[k]);
            if (tb[k] < 400.0 && tb[k] > 180.0 && day >= nb && day < ne) {
	      sum[i][j][k] = sum[i][j][k] + tb[k];
	      num[i][j][k]++;
            }
	  }
    } /* end n loop */
  endofdata: printf("%d %d\n",year,m+1);

    /* compute monthly mean at each grid*/
    for (k=0; k<nch; k++)
      for (i=0; i<nlat; i++) {
	for (j=0; j<nlon; j++) {
	  if (num[i][j][k] > minday)
	    mean[i][j][k] = sum[i][j][k]/num[i][j][k];
	  else
	    mean[i][j][k] = 999.;
	  fprintf(ofp1," %6.2f", mean[i][j][k]);
	}
	fprintf(ofp1,"\n");
      }
    fclose(ifp);

    /* compute monthly global mean */
    for (k=0; k<nch; k++) {
      gsum[k] = 0.0;
      gnum[k] = 0;
      for (i=0; i<nlat; i++)
        for (j=0; j<nlon; j++)
	  if (mean[i][j][k] < 400.0) {
	    gsum[k] = gsum[k] + mean[i][j][k];
	    gnum[k]++;
	  }
      if (gnum[k] > 0) {
	gmean[k] = gsum[k] / gnum[k];
        if (k == 0) {
	  fprintf(ofp2," %7.2f %d", year+m/12.,m+1);
	  fprintf(ofp2," %6.2f", gmean[k]);
        }
	else if (k == nch-1) {
          fprintf(ofp2," %6.2f", gmean[k]);
          fprintf(ofp2,"\n");
        }
        else
          fprintf(ofp2," %6.2f", gmean[k]);
      }
    }
  } /* end m loop */

  fclose(ofp1);
  fclose(ofp2);

}
