c==========================================================================
c@***h*  HIRS/grid_cs2.f
c*
c* NAME
c*  grid_cs2.f
c*
c* LOCATION
c*  /crystal/san3/clearsky4/source/grid_cs2.f
c*
c* PURPOSE
c*  Grids HIRS clear-sky scan line data and bins into grid. Separates
c*  ascending and descending nodes.
c*
c* DESCRIPTION
c*  Grids HIRS clear-sky scan line data and bins into grid. Separates
c*  ascending and descending nodes.
c*
c* AUTHOR
c*  Darren Jackson CIRES/ETL
c*
c* CREATION DATE
c*  Apr 2002
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* MODIFICATION HISTORY
c*  Jan 2010 - D. Wunder put in robodoc format
c*  Sep 2002 - Version 2 computes array of observation times
c*  Sep 2002 - Version 2 identifies asc/des passes
c*  Sep 2002 - Version 2 Adapted from grid_ch4.f
c*
c* INPUTS
c*    day_file.........File name of daily clear-sky file
c*    jday.............julian day of year
c*    iyr4.............4 digit year
c*    nlon.............number of longitude grid points
c*    nlat.............number of latitude grid points
c*    max_cnt..........Maximum number of observations in time array
c*    ichn.............Channel to grid (1-19)
c*    s_range(2).......Range of scan points (1-28)
c*    gres.............grid resolution
c*
c* OUTPUTS
c*    sum..............sum of the values
c*    sumsq............sum squared of values
c*    obs_time.........array of observation times in julian days from Jan 1, 1969.
c*    icnt.............number of points.
c*
c* EXTERNALS
c*  hirs_grid.inc
c*  recl_parms.inc
c*  binit.f
c*  date_to_julian69.f
c*  nodalpass.f
c*
c* NOTES  
c*  No limb correction is available in this code
c*
c* LANGUAGE
c*  Fortran 
c*
c@*****
c============================================================================
      subroutine grid_cs2(day_file,jday,iyr4,nlon,nlat,max_cnt,
     .     ichn,s_range,sum,sumsq,obs_time,icnt,gres)

      implicit none
      include    'hirs_grid.inc'
      include    'recl_parms.inc'

      integer    nlon,nlat,max_cnt,ichn,s_range(2)
      integer    icnt(nlon,nlat,node),ispot(nspot),irec,ii,jj,k
      integer    obs_time(nlon,nlat,max_cnt,node)
      integer    jday,jday69,iyr4
      integer    itmp,ijday69,prvline,prvtime,prvn,n,line

      real       sum(nlon,nlat,node),sumsq(nlon,nlat,node)
      real       alt,fscale,xlon,xlat,gres,ref
      real       parm,prvlon

      character  day_file*85
      logical    bad_data,nolimb

c
c* HIRS CH record
c
c      integer   itime
c      integer*2 ilon,ilat,iszen,ialt,iline,ipct,itc,itsfc
c      byte      isp,iref,iem,iclear
c      integer*2 itb(7),ideltrad(4)
c
c
c* HIRS CS scan line data array (1 record  = 55 bytes) 
c
      integer   itime
      integer*2 ilon,ilat,iszen,ialt,iline
      byte      isp,oqc,csf,iref
      integer*2 itb(nchn2)
c
c* Inititalize
c
      data bad_data/.false./
c
c Initialize first observation as ascending node
c
      n = 1  
c
c* Read and bin scan line data
c
      irec=1
      open(1,file=day_file,access='direct',recl=cs_rec)
 60   read(1,rec=irec,err=99) itime,ilon,ilat,iline,isp,
     .     iszen,ialt,csf,iref,itb

      xlon=fscale(ilon,180.,100.)
      line=iline

      if(isp .ge. s_range(1) .and. isp .le. s_range(2)) then

         call date_to_julian69(iyr4,1,jday,jday69)
         itmp = nint((float(itime) /100. / 86400.) * 1000.)
         ijday69 = jday69 * 1000 + itmp
         xlat=fscale(ilat,0.,100.)
         call binit(gres,0,0,xlon,xlat,ii,jj)
c
c* Do not process first pixel so to find node direction
c
         if(irec .gt. 1) then 

            call nodalpass(itime,xlon,line,n,prvtime,prvlon,
     .           prvline,prvn)
c
c* Convert channel for binning
c     
            parm = fscale(itb(ichn),100.,100.) 
c     
c* Sum resulting parameters
c
            if(parm .gt. -998.) then
               sum(ii,jj,n)=sum(ii,jj,n)+parm
               sumsq(ii,jj,n)=sumsq(ii,jj,n)+parm**2
               icnt(ii,jj,n)=icnt(ii,jj,n)+1
               if(icnt(ii,jj,n) .le. max_cnt) then
                  obs_time(ii,jj,icnt(ii,jj,n),n)=ijday69
               else
                  write(*,*) 'Number of obs. exceeds max_cnt'
                  write(*,*) 'max_cnt,icnt = ',max_cnt,icnt(ii,jj,n) 
                  write(*,*) 'year = ',iyr4,' jday = ',jday
                  write(*,*) 'longitude,latitude = ',xlon,xlat
                  write(*,*) ' '
                  obs_time(ii,jj,1,n) = -1
               endif
            endif

         endif
      endif

      irec=irec+1
      prvn=n
      prvtime=itime
      prvlon=xlon
      prvline=line
      goto 60

 99   close(1)
      write(*,*) 'Read ',irec,' records in ',day_file
      return
      end

