c==========================================================================
c@***h*  HIRS/gridit.f
c*
c* NAME
c*  gridit.f
c*
c* LOCATION
c*  /crystal/san3/clearsky4/source/gridit.f
c*
c* PURPOSE
c*  Read and grid HIRS scan line data for one day
c*
c* DESCRIPTION
c*  Read and grid HIRS scan line data for one day
c*
c* AUTHOR
c*  Darren Jackson CIRES/ETL
c*
c* CREATION DATE
c*  Apr 2002
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* MODIFICATION HISTORY
c*  Jan 2010 - D. Wunder put in robodoc format
c*
c* INPUTS
c*   day_file.........File name of daily clear-sky file
c*   nolimb...........T -> no limb cor., F -> limb correction
c*   rcoef............Regression coefficient array
c*   chnord...........Regression channel order array
c*   npred............Number of predictors array
c*   nlon.............number of longitude grid points
c*   nlat.............number of latitude grid points
c*   nchn.............number of channels
c*   gres.............grid resolution
c*
c* OUTPUTS
c*   sum..............sum of the TB values for each channel and gridpoint
c*   sumsq............sum squared of TB values
c*   icnt.............number of points.
c*
c* EXTERNALS
c*   hirs_grid.nc
c*   binit.f
c*   hlimb.f
c*
c* LANGUAGE
c*  Fortran 
c*
c@*****
c=============================================================================

      subroutine gridit(day_file,nolimb,rcoef,chnord,npred,nlon,nlat,
     .     nchn,sum,sumsq,icnt,gres)

      implicit none
      include    'hirs_grid.inc'

      integer    nlon,nlat,nchn
      integer    icnt(nlon,nlat,nchn),ispot(nspot),irec,ii,jj,k

      real       sum(nlon,nlat,nchn),sumsq(nlon,nlat,nchn)
      real       alt(nspot),fscale,xlon,xlat,gres,ref
      real       tb(nspot,nchn2),tblc(nspot,nchn2)

      character  day_file*80
      logical    bad_data(nspot),nolimb

      integer chnord(ncoef-1,nang,nchn2),npred(nang,nchn2)
      real    rcoef(ncoef,nang,nchn2)
c
c* HIRS scan line data array
c
      integer   itime
      integer*2 ilon,ilat,iszen,ialt,iline
      byte      isp,oqc,csf,iref
      integer*2 itb(nchn2)
c
c* Inititalize
c
      data bad_data/.false./
c
c* Read and bin scan line data
c
      irec=1
      open(1,file=day_file,access='direct',recl=55)
 60   read(1,rec=irec,err=99) itime,ilon,ilat,iline,isp,
     .     iszen,ialt,csf,iref,itb
      xlon=fscale(ilon,180.,100.)
      xlat=fscale(ilat,0.,100.)
      call binit(gres,0,0,xlon,xlat,ii,jj)
c
c* Convert parameters for limb correction
c
      ispot(1)=int(isp)
      alt(1)=fscale(ialt,0.,10.)
      do k=1,nchn2
         tb(1,k)=fscale(itb(k),100.,100.)
      enddo
      ref=iref/100.
c
c* Apply limb correction
c
      if(.not. nolimb) then
         call hlimb(ispot,alt,bad_data,rcoef,chnord,npred,
     .        tb,tblc)
      endif
c
c* Sum resulting TBs (No limb correction for channels 13-19)
c
      do k=1,nchn
         if(k .lt. nchn) then
            if(tb(1,k) .ne. xmiss) then
               if(nolimb .or. k .gt. 12) then
                  sum(ii,jj,k)=sum(ii,jj,k)+tb(1,k)
                  sumsq(ii,jj,k)=sumsq(ii,jj,k)+(tb(1,k))**2
                  icnt(ii,jj,k)=icnt(ii,jj,k)+1
               else
                  if(tblc(1,k) .ne. xmiss) then
                     sum(ii,jj,k)=sum(ii,jj,k)+tblc(1,k)
                     sumsq(ii,jj,k)=sumsq(ii,jj,k)+tblc(1,k)**2
                     icnt(ii,jj,k)=icnt(ii,jj,k)+1
                  endif
               endif
            endif
         else
            if(ref .ge. 0 .and. ref .lt. 2.54) then
              sum(ii,jj,k)=sum(ii,jj,k)+ref
              sumsq(ii,jj,k)=sumsq(ii,jj,k)+ref**2
              icnt(ii,jj,k)=icnt(ii,jj,k)+1
            endif
         endif
      enddo
c      if(irec .eq. 30000) goto 99
      irec=irec+1
      goto 60

 99   close(1)
      write(*,*) 'Read ',irec,' records in ',day_file
      return
      end
