c================================================================================
c@***h*  HIRS/hcloud.f
c*
c* NAME
c*  hcloud.f
c*
c* LOCATION
c*  /crystal/san3/clearsky4/source/hcloud.f
c*
c* PURPOSE
c*  This program reads TOVS1B pixel and gridded data from hirs1c.f data
c*  and applies spatial and time contrast tests for cloud detection.
c*
c* DESCRIPTION
c*  This program reads TOVS1B pixel and gridded data from hirs1c.f data
c*  and applies spatial and time contrast tests for cloud detection.
c*
c* AUTHOR
c*  Darren Jackson CIRES/ETL
c*
c* CREATION DATE
c*  Apr 2002
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* MODIFICATION HISTORY
c*  Jan 2010 - D. Wunder put in robodoc format
c*  Feb 2002 - Lei Shi, Include M02,M01,N18
c*
c* INPUTS
c*   (via hcloud.input)
c*   isat  -> Satellite number (2,5-17)
c*   iyr4  -> 4 digit year
c*   jday  -> day of the year (1-366)
c*   HIRS all-sky swath data  (*.AS)
c*
c* OUTPUTS
c*   HIRS temporary clear-sky swath data (*.CS1)
c*   Adds clear-sky statistics for temporary grid file (*.GRD)
c*
c* EXTERNALS
c*  (see README for description of these routines)
c*   physical_constants.h
c*   hirs1c.inc
c*   open_read.f
c*   get_grid_files.f
c*   swap16.f
c*   binit.f
c*   spatial_contrast.f
c*   time_contrast.f
c*   daygrid.f
c*
c* LANGUAGE
c*  Fortran 
c*
c@*****
c=================================================================================

      program hcloud

      implicit none

      include '../flib/physical_constants.h'          ! physical constants
      include 'hirs1c.inc'                            ! program constants

      byte      lmask_space(nlon,nlat),lmask_time(nlon,nlat)

      real      tovs_hr,radeg
      real      tb_region(nlon,nlat),tb_time(3),pixel
      real      gmxmn(nlon,nlat,ntim,3)
      real      xsmob(nlon,nlat,ntim,cs)

c      integer   oqc
      integer   i,j,k,l
      integer   ierr,iyr4,iom,ion,ioh,iog(3),iof,iol,ioe
      integer   space,time,csf
      integer   jchn
      integer   ii,jj,iii,jjj,ichn(4),it_old
      integer   irec,n,m,it,orec
      integer   isat,jday
      integer*2 elev(3600,1800)

      character satid(16)*3
      character l_file*80

      integer ispot,line
      real    xtime,xlon,xlat,szen,alt,refl,tb(nchn2),tblc(1)
      real    eres
      logical   bad_data
c     
c*  Initialize values
c
      include 'datapaths.inc'

      data eres/0.1/
      data bad_data/.false./
      data iol/10/
      data ioe/11/
      data iom/15/
      data ion/20/
      data iog/30,31,32/
      data iof/40/
      data ioh/41/
      data ichn/8,18,19,20/
      data satid/'N05','N06','N07','N08','N09','N10','N11','N12','N13',
     .           'N14','N15','N16','N17','N18','M01','M02'/

      jchn=1
      radeg=180./pi
      irec=1
      it_old=0
c
c*  Retrieve input parameters
c
      open(10,file='hcloud.input')
      read(10,*) isat
      read(10,*) iyr4
      read(10,*) jday
      close(10)
c
c*  Open all files
c
      if(isat .ge. 5) then
         call open_read(iom,ion,iog,iof,ioh,iol,satid(isat-4),iyr4,
     .               jday,ierr)
      else
         call open_read(iom,ion,iog,iof,ioh,iol,satid(isat+14),iyr4,
     .               jday,ierr)
      endif

      if(ierr .ne. 0) then
         write(ioh,*) 'Can not perform cloud detection for'
         write(ioh,*) 'isat = ',isat,' year = ',iyr4,' jday =',jday
         write(ioh,*) 'Error code = ',ierr
         stop
      endif
c
c*  Establish time period for time comparison
c
      call get_grid_files(iog,gmxmn)
c
c* Retrieve elevation data and swap int*2 data
c
      l_file=misc_path//'elev_0.1d.dat'
      open(ioe,file=l_file,recl=3600*1800*2,access='direct')
      read(ioe,rec=1) elev
      close(ioe)
      do i=1,3600
         do j=1,1800
            call swap16(elev(i,j),1)
         enddo
      enddo

c
c* Input land/sea mask data
c
      read(iol,rec=1) lmask_space
      read(iom,rec=1) lmask_time
c
c*  Loop through each TOVS scan line
c
      orec=1
      irec=1


 80   read(ion,rec=irec,err=85) xtime,xlon,xlat,szen,
     .     alt,ispot,line,refl,tblc,
     .     (tb(n), n=1,nchn2)
c      write(*,*) 'rec:',irec
      write(*,*) irec,refl
c
c* Do not process data poleward of 70 degrees
c
c      if(abs(xlat) .lt. 70.) then
c
c* Find diurnal time index
c
         tovs_hr=xtime/3600.
         it=ifix(tovs_hr/tinc)+1
         if(it .ge. itmax) it=1
c
c* find lat,lon indices
c
         call binit(gres,1,1,xlon,xlat,ii,jj)
c
c* Remove grid boxes with too many observations (navigation error?)
c
         if(gmxmn(ii,jj,it,2) .ne. missing) then
c
c* isolate max/min grid used for spatial and time contrast tests
c
            if(it .ne. it_old) then
               do n=1,nlon
                  do m=1,nlat
                     tb_region(n,m)=gmxmn(n,m,it,2)
                  enddo
               enddo
               it_old=it
            endif
c     
c* night or day
c             nite=.false.
c             if(float(tovs(jj+3)/100.) .ge. 90.) then  ! zenith angle > 90.
c                jchn=3     ! HIRS 19 night
c             else
c                jchn=1     ! HIRS  8 day
c             endif
c
c* spatial constrast test
c
            pixel=tb(ichn(jchn))
            call spatial_contrast(pixel,ichn(jchn),ii,jj,lmask_space,
     .           tb_region,space)
c     
c* Fill tb_time vector for time_constrast routine
c
            do n=1,3
               tb_time(n)=gmxmn(ii,jj,it,n)
            enddo
c
c* time contrast test
c     
            call time_contrast(pixel,ichn(jchn),lmask_time(ii,jj),
     .           tb_time,time)
c     
c* cloud flag parameter
c
            csf=99
            if(time .eq. 3 .and. space .eq. 1) csf=0 ! clear sky
            if(time .eq. 3 .and. space .eq. 2) csf=1 ! coastal clear sky 
c     if(time .eq. 1 .and. space .eq. 1) csf=1 ! undecided
c
c* albedo test
c
            if(csf .lt. 99 .and. lmask_time(ii,jj) .ne. 1 .or. 
     .           (lmask_time(ii,jj) .eq. 1 .and. 
     .           abs(xlat) .le. 30.)) then
               if(refl .gt. 0.6 .and. refl .lt. missing) csf=99
            endif
c     
c* Tropics cold channel 8 Tb test (to remove persistent cloud cover over
c  terrain less than 1000m between 15S and 15N).  Africa is treated as
c  a special case.  Cold Tbs at night can occur and are accounted for here.
c
            if(abs(xlat) .lt. 15. .and.
     .           csf .lt. 99 .and. tblc(1) .lt. 288.) then
               call binit(eres,1,1,xlon,xlat,iii,jjj)
               if(elev(iii,jjj) .lt. 1000) then
                  if(lmask_time(ii,jj) .eq. 0) csf=99
                  if(lmask_time(ii,jj) .gt. 0) then
                     if(xlon .gt. 60. .and. xlon .lt. 340.) csf=99
                     if((xlon .le. 60. .or. xlon .ge. 340.) .and.
     .                    szen .lt. 89.9) csf=99
                  endif
               endif
            endif

            if(csf .lt. 99) then
c
c* Bin limb corrected/cloud corrected TBs for cloud detection into daily files
c
               call daygrid(1,1,bad_data,xtime,xlon,xlat,szen,
     .              refl,tblc,xsmob)
c     
c* output cloud-cleared result
c
               write(iof,rec=orec) xtime,xlon,xlat,szen,
     .              alt,ispot,line,csf,refl,tblc,
     .              (tb(n), n=1,nchn2)
               orec=orec+1
            endif
c        else
c            write(ioh,*) 'Remove obs.: xlon = ',xlon,' xlat = ',xlat
         endif
c      endif
c
c* goto next pixel
c
      irec=irec+1
      goto 80
c
c* Append clear-sky grids to grid file for current day
c
 85   do l=1,cs
         write(iog(2),rec=as+l) (((xsmob(i,j,k,l),i=1,nlon),j=1,nlat),
     .        k=1,ntim)
      enddo
c
c* Close all files
c      
      write(*,*) isat,iyr4,jday,irec-1,orec-1
      close(ion)
      close(iof)
      close(iol)
      close(ioh)
      do n=1,3
         close(iog(n))
      enddo

      end
