c====================================================================================
c@***h*  HIRS/hirs1c.f
c*
c* NAME
c*  hirs1c.f
c*
c* LOCATION
c*  /crystal/san3/clearsky4/source/hirs1c.f
c*
c* PURPOSE
c*  Reads calibrated, geolocated brightness temperatures from ITPP or
c*  AAPP orbit files, performs QC on TB data,  performs limb correction
c*  on channel 8, and writes all-sky swath data file and grid statistics
c*  for cloud detection.
c*
c* DESCRIPTION
c*  Reads calibrated, geolocated brightness temperatures from ITPP or
c*  AAPP orbit files, performs QC on TB data,  performs limb correction
c*  on channel 8, and writes all-sky swath data file and grid statistics
c*  for cloud detection.
c*
c* AUTHOR
c*  Darren Jackson CIRES/ETL
c*
c* CREATION DATE
c*  Apr 2002
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* MODIFICATION HISTORY
c*  Jan 2010 - D. Wunder put in robodoc format
c*
c* INPUTS
c*   (via hirs1c.input file)
c*         nfiles          ->   (INT*4)    number of files to read
c*         orbnum(nfiles)  ->   (CHAR*7)   Orbit ID number for each file
c*         jday            ->   (INT*4)    julian day (day of year 1-366)
c*         iyr             ->   (INT*4)    two digit year
c*         isat            ->   (INT*4)    satellite number (5=TIROS-N,6=NOAA-6,etc.)
c*
c* OUTPUTS
c*     1. pixel level data of cloud-free/limb corrected HIRS data
c*     2. daily grid files of TB max/Vis refl min
c*     3. Orbit statistics data for all-sky data
c*
c* EXTERNALS
c*   (see README for description of their function)
c*   aapp1c.inc
c*   hirs1c.inc   
c*   init_grid.f  
c*   daygrid.f    
c*   open_qc.f
c*   qc_histo.f
c*   open_regcoef.f
c*   open_hirs.f
c*   horb.f
c*   unpack_hirs.f
c*   hqc.f
c*   hlimb1c.f
c*   daygrid.f
c*   open_asd.f
c*   moment.f
c*   maxmin_vec.f
c*
c* NOTES
c*   1.  Could compute orbit statistics but currently turned off.
c*   2.  I may want to reproduce the orbit statistics files since one of the columns
c*       indicating missing/erroneous data always indicates 3000-4000 bad data
c*       points per orbit.  This is due to counting the calibration orbits as bad data.
c*
c* LANGUAGE
c*  Fortran 
c*
c@*****
c=========================================================================
c
c ADDITIONAL NOTES:
c
c     Swath all-sky HIRS data formats:
c
c      file name format - instrument.satellite.year.julian day.V22.AS
c      file type        - unformatted direct access binary
c                         recl=112
c      file record      - time,xlon,xlat,szen,alt,ispot,line,refl,tblc,tb(19)
c                         time   - real*4
c                         xlon   - real*4
c                         xlat   - real*4
c                         szen   - real*4
c                         alt    - real*4
c                         ispot  - int*4
c                         line   - int*4
c                         refl   - real*4
c                         tblc   - real*4
c                         tb     - real*4(19)
c
c     Summed daily grid file formats:
c
c      file name format - instrument.satellite.year.julian day.GRD
c      file type        - unformatted direct access binary
c                           recl=nlon*nlat*nchn*3*2
c      file record      - max(nlon,nlat,nchn),min(nlon,nlat,nchn),
c                             count(nlon,nlat,nchn)
c                         [float(nlon,nlat,nchn)*2,int*4(nlon,nlat,nchn)]
c
c     Orbit statistics formats:
c
c       file name format - 
c       file type        - formatted ASCCI
c       file record      - jmin,orbnum,max,min,ave,sdev,skew,curt,num. obs, num. missing
c
c====================================================================================
c
c  3/5/08  Modified to include N18-19 and M01-02
c
      program hirs1c

      implicit none

      include 'hirs1c.inc'
      include 'aapp1c.inc'

      integer iqc,oqc(nspot)
      integer isat,iyr,jday,nfiles,nblock,irec,ihr,imin,jmin,bad_orb
      integer i,k,l,m,n,iyr4,mon,iday,ic
      integer ion,iog,iok,iou,iol,ierr

      integer head(trec),indata(trec)
      integer ispot(nspot),line(nspot)
      integer ct_orb(nchn),mis_orb(nchn)
      integer chnord(ncoef-1,nang,nchn3),npred(nang,nchn3)
      integer nmiss(nchn),tbcnt(nchn)
c      integer qcct(nchn2,nsat,nyr,nmon)
c      integer qcmn(nchn2,nsat,nyr,nmon),qcsd(nchn2,nsat,nyr,nmon)

      real    time(nspot),xlat(nspot),xlon(nspot),szen(nspot)
      real    lz(nspot),alt(nspot)
      real    xsec,xmax,xmin,ave,adev,sdev,var,curt,skew
      real    tborb(max_allsky_obs,nchn)
c      real    tb(nspot,nchn2),tblc(nspot,1),refl(nspot)
      real    tb(nspot,nchn),tblc(nspot,1),refl(nspot)
      real    rcoef(ncoef,nang,nchn3)
      real    xmxmn(nlon,nlat,ntim,as)

      character orbnum(50)*7,satid(17)*3,orb_file*70
      logical   bad_data(nspot)
c
c* Initialize some variables
c
      include 'datapaths.inc'

      data satid/'N05','N06','N07','N08','N09','N10','N11','N12','N13',
     .           'N14','N15','N16','N17','N18','N19','M01','M02'/
      data ion/30/ ! Temporary all-sky swath file
      data iog/40/ ! daily max/min grid file
      data iok/55/ ! regression coefficient files
      data iou/60/ ! ITPP HIRS processed data
      data iol/10/ ! HIRS orbital stat files
c      data iom/50/ ! Annual HIRS moments from histogram QC analysis
c      data ioq/70/ ! Orbital statistics file from previous run

c
c* Initialize
c
      irec=1
      call init_grid(xmxmn)
c     
c*  Read inputs
c
      open(1,file='hirs1c.input')
      read(1,*) nfiles
      do i=1,nfiles
         read(1,'(a7)') orbnum(i)
      enddo
      read(1,*) jday
      read(1,*) iyr
      read(1,*) isat
      close(1)
c
c
c
      if(isat .gt. 14 .and. nspot .ne. 56 .or. 
     +   isat .lt. 5 .and. nspot .ne. 56) then
         write(*,*) 'nspot needs to be changed to 56'
         write(*,*) 'in hirs1c.inc then recompile hirs1c.f'
         stop
      endif
      if(isat .ge. 5 .and. isat .le. 14 .and. nspot .ne. 4) then
         write(*,*) 'nspot needs to be changed to 4'
         write(*,*) 'in hirs1c.inc then recompile hirs1c.f'
         stop
      endif
c
c* Find month number
c
      if(iyr .lt. 69) then
         iyr4 = iyr + 2000
      else
         iyr4 = iyr + 1900
      endif
      call julian2(jday,iyr4,mon,iday)
c
c* Load HIRS histogram data file
c
c      call qc_histo(iom,qcmn,qcsd,qcct)
c
c* Load HIRS limb regression coefficients
c
      call open_regcoef(iok,isat,rcoef,chnord,npred)
c
c*  Loop through each orbit
c
      do k=1,nfiles
c
c*  Initialize some variables to missing or zero
c
         do n=1,nchn
            tbcnt(n)=0
            nmiss(n)=0
         enddo
c
c*  Open and read header of an orbital file
c
         call open_hirs(isat,iou,k)
         write(*,*) 'iou,isat,k = ',iou,isat,k
c         if(isat .le. 14) then
         if(isat .ge. 5 .and. isat .le. 14) then
            read(iou,rec=1) head
            ihr=int(head(2)/10000.)
            imin=int(head(2)/100.)-(ihr*100)
            jmin=ihr*60+imin
            nblock=(head(4)*56)/4
         else
            read(iou,rec=1) hrs1c_head
            xsec=hrs1c_head.startdatatime/1000.
            jmin=int(xsec/60.)
            nblock=hrs1c_head.scnlin
         endif
c
c*  Orbital QC flag
c
c         call horb(head,ioq,qcmn,qcsd,qcct,iqc)
c         do m=1,nspot
c            oqc(m)=iqc
c         enddo
c
c* Loop through entire orbit
c
         do l=1,nblock
c
c* Unpack hirs input data
c
            call unpack_hirs(l,isat,iou,ispot,line,time,
     .           xlat,xlon,szen,alt,lz,tb,refl,ierr)

c            write(*,*) l,refl

            if(ierr .ne. 0) goto 99
c
c* Apply quality control.  Note that AAPP satellite altitude uses
c  nominal value from header, not altitude value from data record.
c
c            if(isat .ge. 15) then
c               do i=1,nspot
c                  alt(i) = hrs1c_head.satht/10.
c               enddo
c            endif
            call hqc(time,xlon,xlat,alt,lz,line,ispot,tb,bad_data)
c
c* Apply limb correction to channel 8
c
            call hlimb1c(ispot,alt,lz,bad_data,rcoef,chnord,
     .           npred,tb,tblc)
c
c* Test limb corrected HIRS8 for large value
c
            do i=1,nspot
               if(tblc(i,1) .gt. 325.) bad_data(i)=.true.
            enddo
c
c* Bin limb corrected TBs for cloud detection into daily files
c
c            write(*,*) ' '
c            write(*,*) 'block = ',l
c            write(*,*) time(28),xlon(28),xlat(28)
c            write(*,*) ispot(28),line(28)
c            write(*,*) szen(28),alt(28),refl(28)
c            write(*,*) tb(28,8),tblc(28,1)
c            write(*,*) ' '
            call daygrid(0,nspot,bad_data,time,xlon,xlat,szen,
     .           refl,tblc,xmxmn)
c                  write(*,*) l,refl
c
c* Write all-sky swath non-limb TB data and limb corrected channel 8
c
            do m=1,nspot
            write(*,*) irec
               if(.not. bad_data(m) .and. 
     .           (tblc(m,1) .ne. missing)) then
c* Open GRD and AS files if first record
                  if(irec .eq. 1) then
                     if (isat .ge. 5) then
                        call open_asd(ion,iog,satid(isat-4),iyr,jday)
                     else
                        call open_asd(ion,iog,satid(isat+15),iyr,jday)
                     endif
                  endif
                  write(*,*) l,refl(m)
                  write(ion,rec=irec) time(m),xlon(m),xlat(m),szen(m),
     .                 alt(m),ispot(m),line(m),refl(m),
     .                 tblc(m,1),(tb(m,n), n=1,nchn2)
c
c* Compose arrays for orbit statistics
c
                  do ic = 1,nchn2
                     if(tb(m,ic) .lt. missing) then
                        tbcnt(ic) = tbcnt(ic) + 1
                        tborb(tbcnt(ic),ic) = tb(m,ic)
                     else
                        nmiss(ic)=nmiss(ic)+1
                     endif
                  enddo
                  if(refl(m) .lt. missing) then
                     tbcnt(20) = tbcnt(20) + 1
                     tborb(tbcnt(20),20)=refl(m)
                  else
                     nmiss(20)=nmiss(20)+1
                  endif

                  irec=irec+1
               else
                  write(*,*) 'bad data'
                  do ic = 1, nchn
                     nmiss(ic)=nmiss(ic)+1
                  enddo
               endif
               
            enddo

         enddo
c
c* Compute and write orbit statistics
c
         do ic = 1,nchn
            if(tbcnt(ic) .gt. 2) then
               if (isat .ge. 5) then
                  write(orb_file,97) orb_path,'HIRS.',
     .              satid(isat-4),'.C',ic,'.OB'
               else
                  write(orb_file,97) orb_path,'HIRS.',
     .              satid(isat+15),'.C',ic,'.OB'
               endif
               open(iol,file=orb_file,access='append',
     .              status='unknown')
               call moment(tborb(1,ic),tbcnt(ic),ave,adev,
     .              sdev,var,skew,curt)
               call maxmin_vec(tbcnt(ic),tborb(1,ic),xmin,xmax)
               write(iol,60) iyr4,jday,jmin,orbnum(k),tbcnt(ic),
     .              nmiss(ic),xmin,xmax,ave,sdev,skew,curt
               close(iol)
            endif
         enddo
 97      format(a,a5,a3,a2,i2.2,a3)
 60      format(1x,i4,1x,i3,1x,i5,1x,a7,1x,2(i5,1x),
     .        3(f6.2,1x),f5.2,1x,2(f7.2,1x))
c
         goto 90
 99      write(*,88) 'orb# = ',orbnum(k),' # data lines = ',l-1,
     .        'Error in reading ITPP file' 
 88      format(5x,a7,a7,a16,i5,a39)
 90      close(iou)
      enddo
c
c* Message for no data
c
      if(irec .eq. 1) then
         write(*,*) 'No data written to AS or GRD files'
         write(*,*) 'jday = ',jday,' year = ',iyr
         write(*,*) ' '
      endif
c
c* write daily grid data
c
      if(irec .gt. 1) write(iog,rec=1) xmxmn
c
c* close output files
c
      if(irec .gt. 1) then
         close(ion)
         close(iog)
      endif

      end
