c=================================================================================
c@***h*  HIRS/hirs_grid_cs.f
c*
c* NAME
c*  hirs_grid_cs.f
c*
c* LOCATION
c*  /crystal/san3/clearsky4/source/hirs_grid_cs.f
c*
c* PURPOSE
c*  Reads HIRS Ver2.2 clear-sky scan line files and constructs grid files.
c*
c* DESCRIPTION
c*  Reads HIRS Ver2.2 clear-sky scan line files and constructs grid files.
c*
c* AUTHOR
c*  Darren Jackson CIRES/ETL
c*
c* CREATION DATE
c*  Apr 2002
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* MODIFICATION HISTORY
c*  Jan 2010 - D. Wunder put in robodoc format
c*
c* INPUTS
c*   (via hirs_grid_cs.input):
c*     jday......Julian day
c*     iyr4......4 digit Year (ie. 1980)
c*     id........1-daily average,2-pentad average,3-monthly average
c*     isat......NOAA satellite id (4=ALL,5=TN,6=NOAA-6,etc)
c*     nolimb....T - no limb correction
c*               F - limb correction for chns 1-12, no correction for chns 13-20
c*
c*     Input Data
c*     Daily scan line data files are raw binary files (direct access). 
c*     Daily input files must have following name convention:
c*         (instrument.satellite.year.Dxxx.V22.CS)
c*     Examples
c*        HIRS scan line data for NOAA7 for 360 day of 1982 - HIRS.NO7.82.D360.V22.CS
c*
c* OUTPUTS
c*   Data:
c*     This program outputs INTEGER*2 scaled and offset values of the
c*     mean, standard deviation and total # of points.  The scale and offset
c*     can be adjusted in file hirs_grid.inc.  These three variables are outputed
c*     to separate files each having record length = nlon*nlat*nchn*2.
c*
c*     Grid files with name convention:
c*        (instrument.satellite.year.averaging period.resolution.data_type.statistic)
c*     Examples
c*      1980 HIRS 0.5 degree means for NOAA6 for pentad 2 ->
c*                                                       HIRS.N06.80.P02.R05.CS.MN
c*      1992 HIRS 2.5 degree std for NOAA12 for february ->
c*                                                       HIRS.N12.92.M02.R25.CS.SD
c*
c* EXTERNALS
c*  hirs_grid.inc
c*  datapaths.inc
c*  julian2.f
c*  open_regcoef.f
c*  gridit.f
c*  grid_moment_cs.f
c*  write_gridstat.f
c*  two_digit_year.f
c*
c* LANGUAGE
c*  Fortran 
c*
c@*****
c===============================================================================

      program hirs_grid_cs

      implicit none
      include 'hirs_grid.inc'

      integer    nlon,nlat,nchn
      real       gres
      parameter (gres=0.5)
      parameter (nlon=360./gres,nlat=180./gres,nchn=20)

      integer id,jday,iyr4,iyr2,isat,ires,two_digit_year
      integer is,bsat,esat,mn,dy,iday,bday,iok,ics

      integer chnord(ncoef-1,nang,nchn2),npred(nang,nchn2)
      real    rcoef(ncoef,nang,nchn2)

      real    sum(nlon,nlat,nchn),sumsq(nlon,nlat,nchn)
      integer icnt(nlon,nlat,nchn)

      integer*2 imn(nlon,nlat,nchn),isd(nlon,nlat,nchn)
      integer*2 ict(nlon,nlat,nchn)

      real      emday(12),emday_l(12),bmday(12),bmday_l(12)
      character yymm*11,satid(0:13)*3,day_file*80,day*3,cs_path2*70
      logical   leap,pentad,month,around,data_exist,nolimb

      include 'datapaths.inc'

      data satid/'ALL','N05','N06','N07','N08','N09','N10','N11',
     .           'N12','N13','N14','N15','N16','N17'/
      data bmday   / 1,32,60, 91,121,152,182,213,244,274,305,335/
      data emday   /31,59,90,120,151,181,212,243,273,304,334,365/
      data bmday_l / 1,32,61, 92,122,153,183,214,245,275,306,336/
      data emday_l /31,60,91,121,152,182,213,244,274,305,335,366/
      data leap/.false./,pentad/.false./,month/.false./,around/.false./
      data data_exist/.false./
      data iok/10/
c
c*  Input data
c
      open(1,file='hirs_grid_cs.input')
      read(1,*) jday
      read(1,*) iyr4
      read(1,*) id
      read(1,*) isat
      read(1,'(l1)') nolimb
      close(1)
c
c* set bsat and esat
c
      if(isat .eq. 4) then
         bsat=5
         esat=14
      else
         bsat=isat
         esat=isat
      endif
c
c* Compute 2 digit year
c
      iyr2 = two_digit_year(iyr4)
c
c* Find month number
c
      call julian2(jday,iyr4,mn,dy)
c
c* Check resolution parameter
c
      if(gres .eq. 0.5) then
         ires=1
      elseif(gres .eq. 1.0) then
         ires=2
      elseif(gres .eq. 2.0) then
         ires=3
      elseif(gres .eq. 2.5) then
         ires=4
      else
         write(*,*) 'gres needs to reset in hirs_grid.input'
         write(*,*) 'terminate hirs_grid_cs'
         stop
      endif
c
c* check for leap year
c
      if((MOD(iyr4,4) .EQ. 0 .AND. MOD(iyr4,100) .NE. 0)
     .        .OR. MOD(iyr4,400) .EQ. 0) THEN
         leap=.true.
      endif

      if(id .eq. 1) then                         ! output only one day of data
         do is=bsat,esat
            call open_regcoef(iok,is,rcoef,chnord,npred)
            write(day,'(i3.3)') jday
            call julian2(jday,iyr4,mn,dy)
            write(*,*) is,iyr2,mn
            write(yymm,'(a1,i2.2,a2,i2.2,a2,i2.2)') "N",is,".Y",iyr2,
     .           ".M",mn
c            cs_path2=cs_path//yymm//'/'
            cs_path2=cs_path
            ics=index(cs_path2,' ')-1
            write(day_file,99) cs_path2(1:ics),'HIRS.',satid(is-4),
     .           '.Y',iyr2,'.D',day,'.V22.CS'
 99         format(a,a5,a3,a2,i2.2,a2,a3,a7)
            inquire(file=day_file,exist=around)
            if(around) then
               data_exist=.true.
               call gridit(day_file,nolimb,rcoef,chnord,npred,nlon,nlat,
     .              nchn,sum,sumsq,icnt,gres)
            endif
         enddo
         if(data_exist) then
           call grid_moment_cs(nlon,nlat,nchn,sum,sumsq,icnt,imn,
     .        isd,ict)
           call write_gridstat(nlon,nlat,nchn,id,ires,satid(isat-4),
     .          iyr2,jday,imn,isd,ict)
         else
	   write(*,*) 'No data for this time period'
	   write(*,*) 'Last filename = ',day_file
         endif

      elseif(id .eq. 2) then                    ! output one pentad of data
         if((.not. leap) .and. (mod(jday,5) .eq. 0)) pentad=.true.
         if((leap).and.jday.le.55.and.(mod(jday,5).eq.0)) pentad=.true.
         if((leap).and.jday.ge.66.and.(mod(jday-1,5).eq.0))pentad=.true.
         if(pentad) then
            if(leap .and. jday .eq. 61) then
               bday=jday-5
            else
               bday=jday-4
            endif
            do is=bsat,esat
               if(is .ne. 13) then
                  call open_regcoef(iok,is,rcoef,chnord,npred)
                  do iday=bday,jday
                     write(day,'(i3.3)') iday
                    call julian2(iday,iyr4,mn,dy)
                    write(yymm,'(a,i2.2,a,i2.2,a,i2.2)') "N",is,".Y",
     .                   iyr2,".M",mn
c                    cs_path2=cs_path//yymm//'/'
                    cs_path2=cs_path
                    ics=index(cs_path2,' ')-1
                    write(day_file,99) cs_path2(1:ics),'HIRS.',
     .                   satid(is-4),'.Y',iyr2,'.D',day,'.V22.CS'
                    inquire(file=day_file,exist=around)
                    if(around) then
                       data_exist=.true.
                       call gridit(day_file,nolimb,rcoef,chnord,
     .                      npred,nlon,nlat,nchn,sum,sumsq,icnt,gres)
                    endif
                  enddo
               endif
            enddo
            if(data_exist) then
               call grid_moment_cs(nlon,nlat,nchn,sum,sumsq,icnt,imn,
     .              isd,ict)
               call write_gridstat(nlon,nlat,nchn,id,ires,
     .           satid(isat-4),iyr2,jday,imn,isd,ict)
            else
              write(*,*) 'No data for this time period'
              write(*,*) 'Last filename = ',day_file

            endif
         endif

      elseif(id .eq. 3) then                       ! output one month of data
         if((.not. leap).and.(jday .eq. emday(mn))) month=.true.
         if((leap).and.(jday .eq. emday_l(mn))) month=.true.
         write(*,*) jday,leap,mn,emday_l(mn),emday(mn)
         if(month) then
            if(.not. leap) bday=bmday(mn)
            if(leap) bday=bmday_l(mn)
            do is=bsat,esat
               if(is .ne. 13) then
               call open_regcoef(iok,is,rcoef,chnord,npred)
                  do iday=bday,jday
                    write(day,'(i3.3)') iday
                    call julian2(iday,iyr4,mn,dy)
                    write(yymm,'(a1,i2.2,a2,i2.2,a2,i2.2)') "N",is,".Y",
     .                   iyr2,".M",mn
c                    cs_path2=cs_path//yymm//'/'
                    cs_path2=cs_path
                    ics=index(cs_path2,' ')-1
                    write(day_file,99) cs_path2(1:ics),'HIRS.',
     .                   satid(is-4),'.Y',iyr2,'.D',day,'.V22.CS'
                    inquire(file=day_file,exist=around)
                    if(around) then
                       data_exist=.true.
                       call gridit(day_file,nolimb,rcoef,chnord,
     .                     npred,nlon,nlat,nchn,sum,sumsq,icnt,gres)
                    endif
                 enddo
              endif
            enddo
            if(data_exist) then
               call grid_moment_cs(nlon,nlat,nchn,sum,sumsq,icnt,imn,
     .              isd,ict)
               call write_gridstat(nlon,nlat,nchn,id,ires,satid(isat-4),
     .              iyr2,jday,imn,isd,ict)
            else
               write(*,*) 'No data for this time period'
               write(*,*) 'Last filename = ',day_file
            endif
         else
            write(*,*) 'Month parameter is false'
         endif
      endif

      end






