c====================================================================================
c@***h*  HIRS/hirs_grid_cs2.f
c*
c* NAME
c*  hirs_grid_cs2.f
c*
c* LOCATION
c*  /crystal/san3/clearsky4/source/hirs_grid_cs2.f
c*
c* PURPOSE
c*  Reads HIRS clear-sky swath files and constructs the grid files.
c*  Writes observation times for each grid box.
c*  Writes grids for ascending/descending nodes.
c*
c* DESCRIPTION
c*  Reads HIRS clear-sky swath files and constructs the grid files.
c*  Writes observation times for each grid box.
c*  Writes grids for ascending/descending nodes.
c*
c* AUTHOR
c*  Darren Jackson CIRES/ETL
c*
c* CREATION DATE
c*  Apr 2002
c*
c* COPYRIGHT
c*  THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*  THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*  AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*  AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*  SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*  THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*  TO USERS.
c*
c* MODIFICATION HISTORY
c*  Jan 2010 - D. Wunder put in robodoc format
c*  Jan 2004 - Adapted from hirs_grid_ch4.f
c*  Jan 2004 - Version 2, adds ability for grids of asc/des nodes
c*
c* INPUTS
c*   (via hirs_grid_ch4.input):
c*     jday...... Julian day
c*     iyr4...... 4 digit Year (ie. 1980)
c*     id........ 1-daily average,2-pentad average,3-monthly average
c*     isat...... NOAA satellite id (4=ALL,5=TN,6=NOAA-6,etc)
c*     iparm..... Parameter type
c*                  1  = cloud top pressure (mb)
c*                  2  = cloud top temperature (K)
c*                  3  = effective emissivity
c*                  4  = Tb channel 4 (K)
c*                  5  = Tb channel 5 (K)
c*                  6  = Tb channel 6 (K)
c*                  7  = Tb channel 7 (K)
c*                  8  = Tb channel 8 (K)
c*                  10 = Tb channel 10 (K)
c*                  12 = Tb channel 12 (K)
c*                  20 = Surface temperature (K)
c*                  24 = Rclr-Rcld channel 4
c*                  25 = Rclr-Rcld channel 5
c*                  26 = Rclr-Rcld channel 6
c*                  27 = Rclr-Rcld channel 7
c*                  30 = total cloud frequency (%)
c*                  31 = high cloud frequency (%)
c*     angle..... cosine of scan angle (10=1.0-0.95, 09=0.95-0.85,
c*                                      08=0.85-0.75,07=0.75-0.65)
c*     s_range... start and end scan spots away from nadir (1-28)
c*
c*     Input File
c*              *.CS clear-sky swath files
c*     Daily input files must have following name convention:
c*                (instrument.satellite.year.day.CS)
c*
c* OUTPUTS
c*     This program outputs INTEGER*2 scaled and offset values of the
c*     mean, standard deviation and total # of points.  The scale and offset
c*     can be adjusted in file msu.inc.  These three variables are outputed
c*     to separate files each having record length = nlon*nlat*nparm*2.
c*
c*     Statistic name table:  Txx = HIRS channel x (x=4-8,10,12)
c*                            CP  = Cloud top pressure
c*                            CT  = Cloud top temperature
c*                            EM  = effective emissivity
c*                            TS  = surface temperature from NCEP Reanalysis
c*                            DRx = Rclr-Rcld signal (x=4-7)
c*                            FR  = Cloud frequency
c*
c*     Grid files with name convention:
c*        instrument.satellite.year.averaging period.resolution.
c*                   .<file angle> or <spot range>.CH parameter.node.statistic.
c*                   .clear/cloudy data (CS or AS)
c*     Examples
c*      1980 HIRS 0.5 degree emissivity pentad 2 average for NOAA6, angle 1.0-0.95 
c*        and 6Z  -> HIRS.N06.Y80.P02.R05.06Z.A10.EM.A.MN.CS
c*      1983 HIRS 1.0 degreee descend nodes only
c*                   HIRS.N08.Y83.M12.R10.S02.T04.D.SD.CS
c*
c* EXTERNALS
c*   hirs_grid.inc
c*   datapath.inc 
c*   julian2.f
c*   grid_cs2.f
c*   grid_moment_cs2.f
c*   write_gridstat5.f
c*   two_digit_year.f
c*
c* LANGUAGE
c*  Fortran 
c*
c@*****
c==================================================================================
c
      program hirs_grid_cs2

      implicit none
      include 'hirs_grid.inc'

      integer    nlon,nlat,nparm,max_cnt
      real       gres
      parameter (gres=2.5,max_cnt=500)
      parameter (nlon=360./gres,nlat=180./gres)

      integer id,jday,iyr4,iyr2,isat,ires,ich,two_digit_year
      integer is,bsat,esat,mn,dy,iday,bday,iok,iparm
      integer s_range(2)

      real    sum(nlon,nlat,node),sumsq(nlon,nlat,node)
      integer obs_time(nlon,nlat,max_cnt,node)
      integer icnt(nlon,nlat,node)

      integer*2 imn(nlon,nlat,node),isd(nlon,nlat,node)
      integer*2 ict(nlon,nlat,node)

      real      emday(12),emday_l(12),bmday(12),bmday_l(12)
      character yymm*11,satid(0:12)*3,day_file*85,day*3,cs_path2*70
      character angle*2
      logical   leap,pentad,month,around,data_exist,nolimb

      include 'datapaths.inc'

      data satid/'ALL','N05','N06','N07','N08','N09','N10','N11',
     .           'N12','N13','N14','N15','N16'/
      data bmday   / 1,32,60, 91,121,152,182,213,244,274,305,335/
      data emday   /31,59,90,120,151,181,212,243,273,304,334,365/
      data bmday_l / 1,32,61, 92,122,153,183,214,245,275,306,336/
      data emday_l /31,60,91,121,152,182,213,244,274,305,335,366/
      data leap/.false./,pentad/.false./,month/.false./,around/.false./
      data data_exist/.false./
      data iok/10/
c
c*  Input data
c
      open(1,file='hirs_grid_cs2.input')
      read(1,*) jday
      read(1,*) iyr4
      read(1,*) id
      read(1,*) isat
      read(1,*) iparm
      read(1,'(a2)') angle
      read(1,*) s_range
      close(1)

c
c* set bsat and esat
c
      if(isat .eq. 4) then
         bsat=5
         esat=14
      else
         bsat=isat
         esat=isat
      endif
c
c* Compute 2 digit year
c
      iyr2 = two_digit_year(iyr4)
c
c* Find month number
c
      call julian2(jday,iyr4,mn,dy)
c
c* Check resolution parameter
c
      if(gres .eq. 0.5) then
         ires=1
      elseif(gres .eq. 1.0) then
         ires=2
      elseif(gres .eq. 2.0) then
         ires=3
      elseif(gres .eq. 2.5) then
         ires=4
      else
         write(*,*) 'gres needs to reset in hirs_grid.input'
         write(*,*) 'terminate hirs_grid_ch'
         stop
      endif
c
c* check for leap year
c
      if((MOD(iyr4,4) .EQ. 0 .AND. MOD(iyr4,100) .NE. 0)
     .        .OR. MOD(iyr4,400) .EQ. 0) THEN
         leap=.true.
      endif

      if(id .eq. 1) then                         ! output only one day of data
         do is = bsat,esat
            call julian2(jday,iyr4,mn,dy)
            write(yymm,'(a1,i2.2,a2,i2.2,a2,i2.2)') "N",is,".Y",
     .           iyr2,".M",mn
            cs_path2=cs_path//yymm//'/'
            ich=index(cs_path2,' ')-1
            
            write(day_file,99) cs_path2(1:ich),'HIRS.',
     .           satid(is-4),'.Y',iyr2,'.D',jday,'.V22.CS'
            inquire(file=day_file,exist=around)
            if(around) then
               data_exist=.true.
               call grid_cs2(day_file,jday,iyr4,nlon,nlat,max_cnt,
     .              iparm,s_range,sum,sumsq,obs_time,icnt,gres)
            endif
         enddo
         if(data_exist) then
           call grid_moment_cs2(nlon,nlat,sum,sumsq,icnt,
     .           imn,isd,ict)
           call write_gridstat5(nlon,nlat,max_cnt,id,ires,iparm,s_range,
     .          satid(isat-4),iyr2,jday,imn,isd,ict,obs_time)
         else
	   write(*,*) 'No data for this time period'
	   write(*,*) 'Last filename = ',day_file
         endif

      elseif(id .eq. 2) then                    ! output one pentad of data
         if((.not. leap) .and. (mod(jday,5) .eq. 0)) pentad=.true.
         if((leap).and.jday.le.55.and.(mod(jday,5).eq.0)) pentad=.true.
         if((leap).and.jday.ge.66.and.(mod(jday-1,5).eq.0))pentad=.true.
         if(pentad) then
            if(leap .and. jday .eq. 61) then
               bday=jday-5
            else
               bday=jday-4
            endif
            do is=bsat,esat
               if(is .ne. 13) then
                  do iday=bday,jday
                    call julian2(iday,iyr4,mn,dy)
                    write(yymm,'(a1,i2.2,a2,i2.2,a2,i2.2)') "N",is,".Y",
     .                   iyr2,".M",mn
                    cs_path2=cs_path//yymm//'/'
                    ich=index(cs_path2,' ')-1
                    write(day_file,99) cs_path2(1:ich),'HIRS.',
     .                   satid(is-4),'.Y',iyr2,'.D',iday,'V22.CS'
                    inquire(file=day_file,exist=around)
                    if(around) then
                       data_exist=.true.
                       call grid_cs2(day_file,iday,iyr4,nlon,nlat,
     .                      max_cnt,iparm,s_range,sum,sumsq,obs_time,
     .                      icnt,gres)
                    endif
                enddo
               endif
            enddo
            if(data_exist) then
               call grid_moment_cs2(nlon,nlat,sum,sumsq,
     .              icnt,imn,isd,ict)
               call write_gridstat5(nlon,nlat,max_cnt,id,ires,iparm,
     .              s_range,satid(isat-4),iyr2,jday,imn,isd,ict,
     .              obs_time)
            else
              write(*,*) 'No data for this time period'
              write(*,*) 'Last filename = ',day_file

            endif
         endif

      elseif(id .eq. 3) then                       ! output one month of data
         if((.not. leap).and.(jday .eq. emday(mn))) month=.true.
         if((leap).and.(jday .eq. emday_l(mn))) month=.true.
c         write(*,*) jday,leap,mn,emday_l(mn),emday(mn)
         if(month) then
            if(.not. leap) bday=bmday(mn)
            write(*,*) 'mn,bmday = ',mn,bmday(mn)
            if(leap) bday=bmday_l(mn)
            do is=bsat,esat
               if(is .ne. 13) then
                  do iday=bday,jday
c                     write(*,*) "iday = ",iday
                    call julian2(iday,iyr4,mn,dy)
c                    write(*,*) "mn,dy = ",mn,dy
                    write(yymm,'(a1,i2.2,a2,i2.2,a2,i2.2)') "N",is,".Y",
     .                   iyr2,".M",mn
                    cs_path2=cs_path//yymm//'/'
                    ich=index(cs_path2,' ')-1
                    write(day_file,99) cs_path2(1:ich),'HIRS.',
     .                   satid(is-4),'.Y',iyr2,'.D',iday,'.V22.CS'
 99                 format(a,a5,a3,a2,i2.2,a2,i3.3,a7)
                    inquire(file=day_file,exist=around)
                    if(around) then
                       data_exist=.true.
                       call grid_cs2(day_file,iday,iyr4,nlon,nlat,
     .                      max_cnt,iparm,s_range,sum,sumsq,obs_time,
     .                      icnt,gres)
                    endif
                enddo
               endif
            enddo
            if(data_exist) then
               call grid_moment_cs2(nlon,nlat,sum,sumsq,
     .              icnt,imn,isd,ict)
               call write_gridstat5(nlon,nlat,max_cnt,id,ires,iparm,
     .              s_range,satid(isat-4),iyr2,jday,imn,isd,ict,
     .              obs_time)
            else
               write(*,*) 'No data for this time period'
               write(*,*) 'Last filename = ',day_file
            endif
         else
            write(*,*) 'Month parameter is false'
         endif
      endif

      end






